/********************************************************************************/
/*    H   H Y   Y TTTTT EEEEE  CCC       HYTEC ELECTRONICS LTD                  */
/*    H   H  Y Y    T   E     C          5 Cradock Road,                        */
/*    HHHHH   Y     T   EEE   C          Reading, Berks.    Tel: 0118 9757770   */
/*    H   H   Y     T   E     C          RG2 0JT            Fax: 0118 9757566   */
/*    H   H   Y     T   EEEEE  CCC       Web: www.hytec-electronics.co.uk       */
/********************************************************************************/

/* $Author: cross $ */
/* $Date $ */
/* $Id: devHy8403.c,v 1.1.1.1 2005/10/26 08:45:02 cross Exp $ */
/* $Name:  $ */
/* $Revision: 1.1.1.1 $ */
/* $Log $ * */

/********************************************************************************/
/*                                                                              */
/*  drvHy8403ip.c - Driver interface for Hytec 8403 ADC industry pack           */
/*                                                                              */
/*  (C)2004 Hytec Electronics Ltd.                                              */
/*                                                                              */
/********************************************************************************/
/*                                                                              */
/*  Revision history: (comment and initial revisions)                           */
/*                                                                              */
/*  vers.       revised         modified by                                     */
/*  -----       -----------     -------------------------                       */
/*  See Above.  24-OCT-2005     Darrell Nineham Hytec Electronics Ltd.          */
/*                              1. Upgraded to Hytec Coding Standard.           */
/*                              2. Copied from Non-Hot Swap Version.            */
/*                                                                              */
/*                                                                              */
/* ------------------- Copied from Non - Hot Swap Version --------------------  */
/*                                                                              */
/*                                                                              */
/*  1.3         02-MAY-2002     Steve Hunt                                      */
/*                              1. Copy of last Non Hot Swap Version.           */
/*                                                                              */
/********************************************************************************/

#include <vxWorks.h>
#include <stdlib.h>
#include <types.h>
#include <stdioLib.h>
#include <string.h>
#include <intLib.h>

#include <alarm.h>
#include <dbDefs.h>
#include <dbAccess.h>
#include <recSup.h>
#include <devSup.h>
#include <dbScan.h>
#include <aiRecord.h>
#include <waveformRecord.h>

#include <drvHy8403.h>

STATIC long get_ioinit_info(); 

static int debug=0;

/*******************************  Waveform Declarations *************/

struct wf_dev_sup {
	long            number;
	DEVSUPFUN       dev_report;
	DEVSUPFUN       init;
	DEVSUPFUN       init_record;
	DEVSUPFUN       get_ioint_info;
	DEVSUPFUN       read_wf;
};

STATIC long init_record_wf();
STATIC long read_wf();

struct wf_dev_sup WF_DEV_SUP = {
	5,
	NULL,
	NULL,
	init_record_wf,
	get_ioinit_info,
	read_wf
};

/*******************************  Analogue Input Declarations *************/

struct ai_dev_sup {
	long            number;
	DEVSUPFUN       dev_report;
	DEVSUPFUN       init;
	DEVSUPFUN       init_record;
	DEVSUPFUN       get_ioint_info;
	DEVSUPFUN       read_ai;
   	DEVSUPFUN       special_linconv; 
};

STATIC long init_record_ai();
STATIC long read_ai();
STATIC long linconv_ai();

/*struct ai_dev_sup AI_DEV_SUP = {*/
struct ai_dev_sup BLABLA = {
	6,
	NULL,
	NULL,
	init_record_ai,
	get_ioinit_info,
	read_ai,
	linconv_ai,
};


/*********************  Analogue Input Definition *************/

STATIC long init_record_ai( struct aiRecord *pai) {
  struct vmeio *pvmeio = (struct vmeio *) &(pai->inp.value);
  int status;
  unsigned long sVal;
  switch (pai->inp.type) {
  case (VME_IO):
    printf("init_record_ai \n");
    break;
  default:
    recGblRecordError(S_db_badField, (void *) pai,
		      "devAiSSI (init_record) Illegal INP field");
    return(S_db_badField);
   }

  pai->eslo = (pai->eguf - pai->egul) / 16777216.0;

  status = READ_AI(pvmeio->card,
		   pvmeio->signal, pvmeio->parm,
		   &sVal);
  if (status != S_drv_OK) {
    recGblSetSevr(pai, READ_ALARM, INVALID_ALARM);
  }
  return(Ok);
}


STATIC long read_ai(struct aiRecord *pai) {
  struct vmeio *pvmeio;/*  = &pai->inp.value.vmeio; */
  int status;
  unsigned long sVal;
  /*begin*/
  if (debug)
    printf("read_ai\n");

  pvmeio=(struct vmeio *)&(pai->inp.value);
  /*status=READ_AI(pvmeio->card, pvmeio->signal,
    pvmeio->parm,(unsigned short *) &sVal); */

  status=READ_AI(pvmeio->card,
		 pvmeio->signal,pvmeio->parm,&sVal);
  if (status != S_drv_OK) {
    recGblSetSevr(pai, READ_ALARM, INVALID_ALARM);
    if (debug)printf("read_ai : error in read \n");
  }
  pai->rval =  sVal;
  return(status); 
}


STATIC long get_ioinit_info(int cmd, struct aiRecord *pai, IOSCANPVT *ppvt) {
  /*   getioscanpvt(pai->inp.value.vmeio.card,ppvt);*/
  get8403ioscanpvt(pai->inp.value.vmeio.card,ppvt);
  if (debug)printf("get_ioinit_info : \n");
  /*
    getioscanpvt(pbi->inp.value.vmeio.card,ppvt);
    printf("get_ioinit_info : ppvt = %p\n",*ppvt);
    *ppvt = ioscanpvt;
    */
  return 0;
}


STATIC long linconv_ai( struct aiRecord *pai, int after) {
  struct vmeio *pvmeio = (struct vmeio *) &(pai->inp.value);
  if (!after)
    return(Ok);  
  
  pai->eslo = (pai->eguf - pai->egul) / 256.0;
  return(Ok);
}
 

/*********************  waveform  Definition *************/

STATIC long init_record_wf(struct waveformRecord *pwf) {
  int status;
  unsigned short sVal[8192]; 
  struct vmeio *pvmeio = (struct vmeio *) &(pwf->inp.value); 

  switch (pwf->inp.type) {
  case (VME_IO):
    break;
  default:
    recGblRecordError(S_db_badField, (void *) pwf,
		      "devWf (init_record) Illegal INP field");
    return(S_db_badField);
  }
  
  status=READ_WF(pvmeio->card, pvmeio->signal,
		 pvmeio->parm, (unsigned short *) sVal,pwf->nelm); 
  if (status != S_drv_OK) {
    recGblSetSevr(pwf, READ_ALARM, INVALID_ALARM);
  }
  return(Ok);
}


STATIC long read_wf(struct waveformRecord *pwf) {
  int status=0;
  struct vmeio *pvmeio = (struct vmeio *)&(pwf->inp.value);

  pvmeio = (struct vmeio *)&(pwf->inp.value);
  status = READ_WF(pvmeio->card, pvmeio->signal,pvmeio->parm,(unsigned short *) pwf->bptr,pwf->nelm); 
  if (status != S_drv_OK) {
    recGblSetSevr(pwf, READ_ALARM, INVALID_ALARM);
    /*  	printf("read_wf : error in read_wf \n");*/
  }
  pwf->nord=pwf->nelm;
  return(status); 
}

 
/*************************************************************************/
/*                                                                       */
/*                         devHy8402.c                                 */
/*                                                                       */
/*************************************************************************/

/* $Author: cross $ */ 
/* $Date: 2005/10/26 08:45:02 $ */ 
/* $Id: devHy8403.c,v 1.1.1.1 2005/10/26 08:45:02 cross Exp $ */  
/* $Name:  $ */ 
/* $Revision: 1.1.1.1 $ */ 

static char cvsid[]="$Id: devHy8403.c,v 1.1.1.1 2005/10/26 08:45:02 cross Exp $";

#include <vxWorks.h>
#include <stdlib.h>
#include <types.h>
#include <stdioLib.h>
#include <string.h>

#include <alarm.h>
#include <dbDefs.h>
#include <dbAccess.h>
#include <recSup.h>
#include <devSup.h>
#include <dbScan.h>
#include <aoRecord.h>

#include <drvHy8403.h>



/*******************************  Analogue Output Declarations ******/
struct ao_dev_sup {
  long            number;
  DEVSUPFUN       report;
  DEVSUPFUN       init;
  DEVSUPFUN       init_record;
  DEVSUPFUN       get_ioint_info;
  DEVSUPFUN       write_ao;
  DEVSUPFUN       special_linconv;
};

STATIC long init_record_ao();
STATIC long write_ao();
STATIC long linconv_ao();

struct ao_dev_sup BLUBLU = {
  6,
  NULL,
  NULL,
  init_record_ao,
  NULL,
  write_ao,
  linconv_ao
};


/*******************************  Analogue Output Definition *******/

STATIC long init_record_ao(struct aoRecord *pao) {
  if (debug) printf("init_record_ao\n");
  switch (pao->out.type) {
  case (VME_IO):
    break;
  default:
    recGblRecordError(S_db_badField, (void *) pao,
		      "devAo (init_record) Illegal INP field");
    printf("init_record_ao: ERROR not VME\n");
    return(S_db_badField);
  }
  pao->eslo = (pao->eguf - pao->egul) / 256.0;
  pao->roff = 0.0;
  return(2);
}

STATIC long write_ao(struct aoRecord *pao) {
   struct vmeio *pvmeio = &pao->out.value.vmeio;
   int status;
   status = WRITE_AO(pvmeio->card, pvmeio->signal, pao->rval);
   if (status != S_drv_OK) {
     recGblSetSevr(pao, WRITE_ALARM, INVALID_ALARM);
     if (debug) printf("write_ao not OK\n");
   }
   return(status);
}

STATIC long linconv_ao( struct aoRecord *pao, int after) {
  if (!after)
    return(Ok);
  pao->eslo = (pao->eguf - pao->egul) / 256.0;
  pao->roff = 0.0;
  return(Ok);
}




