/********************************************************************************/
/*    H   H Y   Y TTTTT EEEEE  CCC       HYTEC ELECTRONICS LTD                  */
/*    H   H  Y Y    T   E     C          5 Cradock Road,                        */
/*    HHHHH   Y     T   EEE   C          Reading, Berks.    Tel: 0118 9757770   */
/*    H   H   Y     T   E     C          RG2 0JT            Fax: 0118 9757566   */
/*    H   H   Y     T   EEEEE  CCC       Web: www.hytec-electronics.co.uk       */
/********************************************************************************/

/* $Author: cross $*/
/* $Date: 2005/11/24 15:54:40 $ */
/* $Id: drvHy8403.c,v 1.2 2005/11/24 15:54:40 cross Exp $ */
/* $Name:  $ */
/* $Revision: 1.2 $*/

/********************************************************************************/
/*                                                                              */
/*  drvHy8403ip.c - Driver interface for Hytec 8403 ADC industry pack           */
/*                                                                              */
/*  (C)2004 Hytec Electronics Ltd.                                              */
/*                                                                              */
/********************************************************************************/
/*                                                                              */
/*  Revision history: (comment and initial revisions)                           */
/*                                                                              */
/*  vers.       revised         modified by                                     */
/*  -----       -----------     -------------------------                       */
/*  See Above   21-NOV-2005     Darrell Nineham Hytec Electronics Ltd.          */
/*                              1. Improved PT-100 Reading Accuracy.            */
/*                              2. Added ability for READ_AI to access          */
/*                              Thermocouple Calibration Values.                */
/*                                                                              */
/*  1.1.1.1     24-OCT-2005     Darrell Nineham Hytec Electronics Ltd.          */
/*                              1. Added an additional aitype (3) to support    */
/*                              Thermocouple via Hy8913 Din Rail Break out Box. */
/*                              2. Moved to Hy8403 Directory as now Hot Swap.   */
/*                                                                              */
/*                                                                              */
/* ------------------- Copied from Non - Hot Swap Version --------------------  */
/*                                                                              */
/*                                                                              */
/*  1.7			01-OCT-2004     Darrell Nineham Hytec Electronics Ltd.          */
/*                              1. Added an additional aitype (2) to support    */
/*                              PT-100 via Hy8913-P Din Rail Break out Box.     */
/*                              2. Upgraded to Hytec Coding Standard.           */
/*                                                                              */
/*  1.1         27-JUL-2002     Steve Hunt                                      */
/*                              Initial Version ?                               */
/*                                                                              */
/********************************************************************************/

/* ANSI standard */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include <semaphore.h>

/* OS specific vxworks */
#include <vxWorks.h>
#include <vxLib.h>
#include <vme.h>
#include <rebootLib.h>
#include <taskLib.h>
#include <sysLib.h>
#include <logLib.h>
#include <intLib.h>
#include <iv.h>
#include <tickLib.h>
#include <time.h>

/* EPICS specific */
#include <drvSup.h>
#include <drvIpac.h>

/* driver specific */
#include "drvHy8403.h"

typedef unsigned char byte;


typedef unsigned short word;
#define BITSPERWORD  16

typedef unsigned int longword;
#define MAXLONGWORD 0xFFFFFFFF


/* the number of DAC channels per card */
#define NUMAOCHAN  2


/* Xilinx register offsets */
#define REG_CSR     0x00
#define REG_CONVP   0x02
#define REG_NCONV   0x04
#define REG_CLKRAT  0x06
#define REG_INTVEC  0x08
#define REG_ADCCON  0x0A
#define REG_ADCDAT  0x0C
#define REG_ACR     0x0E
#define REG_DACOUT  0x10


#define REG_PGAMUX  0x20

#define REG_PGAMUX_ADCA_1  (REG_PGAMUX)
#define REG_PGAMUX_ADCB_1  (REG_PGAMUX + 2)
#define REG_PGAMUX_ADCA_2  (REG_PGAMUX + 4)
#define REG_PGAMUX_ADCB_2  (REG_PGAMUX + 6)
#define REG_PGAMUX_ADCA_3  (REG_PGAMUX + 8)
#define REG_PGAMUX_ADCB_3  (REG_PGAMUX + 10)
#define REG_PGAMUX_ADCA_4  (REG_PGAMUX + 12)
#define REG_PGAMUX_ADCB_4  (REG_PGAMUX + 14)
#define REG_PGAMUX_ADCA_5  (REG_PGAMUX + 16)
#define REG_PGAMUX_ADCB_5  (REG_PGAMUX + 18)
#define REG_PGAMUX_ADCA_6  (REG_PGAMUX + 20)
#define REG_PGAMUX_ADCB_6  (REG_PGAMUX + 22)
#define REG_PGAMUX_ADCA_7  (REG_PGAMUX + 24)
#define REG_PGAMUX_ADCB_7  (REG_PGAMUX + 26)
#define REG_PGAMUX_ADCA_8  (REG_PGAMUX + 28)
#define REG_PGAMUX_ADCB_8  (REG_PGAMUX + 30)

#define THERMOCOUPLE_CAL_A REG_PGAMUX_ADCA_8
#define THERMOCOUPLE_CAL_B REG_PGAMUX_ADCB_7
#define THERMOCOUPLE_PTR   REG_PGAMUX_ADCB_8

/* Define Required for Calibrating Thermocouples */
#define CALBITS	7110893



/* 
    REG_PGAMUX registers are 16 registers of 16 bits each.
    Each register describes the ADC multiplexer setting
    for the given channel.
*/

#define REG_ADCBASE 0x40
/*
    the ADC data registers are 16 channels of 32 bits
    starting from REG_ADCBASE (0x40 to 0x7E).
    chan 1 lo
    chan 1 hi
    chan 2 lo
    chan 2 hi
    ...
    chan 16 lo
    chan 16 hi
*/



/* define the bits in the CSR register */
#define CSR_RDY  0x0001
#define CSR_F    0x0002
#define CSR_CC   0x0004
#define CSR_MIS  0x0008
#define CSR_MII  0x0010
#define CSR_EII  0x0020
#define CSR_ECAL 0x0040
#define CSR_1M   0x0080
#define CSR_HE   0x0100
#define CSR_FE   0x0200
#define CSR_EE   0x0400
#define CSR_ET   0x0800
#define CSR_XC   0x1000
#define CSR_ST   0x2000
#define CSR_EX   0x4000
#define CSR_ARM  0x8000

/* 
    define the opcodes that can be written to the ADC control
    register REG_ADCCON
*/

#define CON_ADCA           0x0000
#define CON_ADCB           0x8000

#define CON_WREG8          0x0001
#define CON_WREG88         0x0101
#define CON_RREG8          0x0002
#define CON_RREG88         0x0102
#define CON_RDAT           0x0013
#define CON_CALOG          0x0004
#define CON_CALO           0x0014
#define CON_CALG           0x0024
#define CON_SCALO          0x0034
#define CON_SCALG          0x0044
#define CON_SYNC           0x00C4
#define CON_SLEEP          0x00D4
#define CON_RESET          0x00E4
#define CON_REG2RAM        0x00A5
#define CON_RAM2REG        0x00A6


/*
    this is how much memory to reserve for each channel
    in word space.
    We have space for a total of 512K 32 bit samples;

    single ended (16 channels):
    1MB: 16K samples = 32K words
    2MB: 32K samples = 64K words

    differential (8 channels):
    1MB: 32K samples = 64K words
    2MB: 64K samples = 128K words
*/
#define MEM_SINGLE_1MB   0x08000
#define MEM_SINGLE_2MB   0x10000

#define MEM_DIFF_1MB     0x10000
#define MEM_DIFF_2MB     0x20000


/*
    we have 16 bytes of configuration registers on the ADC.
    I can store these as 8 words
*/

#define NUMADCWORD   8

/*
    Each register in the ADC has a number [0..15].
    These numbers are used when issuing commands to the
    ADC (read, write, etc.)
*/


#define ADC_SETUP  (0x00)
#define ADC_MUX    (0x01)
#define ADC_ACR    (0x02)
#define ADC_IDAC1  (0x03)
#define ADC_IDAC2  (0x04)
#define ADC_ODAC   (0x05)
#define ADC_DIO    (0x06)
#define ADC_DIR    (0x07)
#define ADC_DEC0   (0x08)
#define ADC_DEC1   (0x09)
#define ADC_OCRO   (0x0A)
#define ADC_OCR1   (0x0B)
#define ADC_OCR2   (0x0C)
#define ADC_FSR0   (0x0D)
#define ADC_FSR1   (0x0E)
#define ADC_FSR2   (0x0F)
#define NUMADCREG  0x10


/* ADC SETUP Register Bit Defines */
/* ------------------------------ */
#define ADC_SETUP_BORD	0x01	/* Bit Order 0 - MSB First, 1 - LSB First */
#define ADC_SETUP_BUFEN	0x02	/* Buffer Enabled */ 
#define ADC_SETUP_REFHI	0x04	/* Internal Ref 0 - 1.25V, 1 - 2.5V */ 
#define ADC_SETUP_REFEN	0x08	/* Internal Ref 0 - Disabled, 1 - Enabled */
#define ADC_SETUP_SPEED	0x10	/* Speed 0 - FMOD = 128, 1 - FMOD = 256 */
#define ADC_SETUP_ID1	0x20	/* Factory Set Bits */
#define ADC_SETUP_ID2	0x40
#define ADC_SETUP_ID3	0x80


/* ADC ODAC Register Bit Defines */
/* ----------------------------- */
#define ADC_ODAC_OSET0	0x01	/*  */
#define ADC_ODAC_OSET1	0x02	/*  */ 
#define ADC_ODAC_OSET2	0x04	/*  */ 
#define ADC_ODAC_OSET3	0x08	/*  */
#define ADC_ODAC_OSET4	0x10	/*  */
#define ADC_ODAC_OSET5	0x20	/*  */
#define ADC_ODAC_OSET6	0x40	/*  */
#define ADC_ODAC_SIGN	0x80	/*  */

#define ADC_ODAC_OFFSET_VOLTAGE	0xFF


/* ADC DEC0 Register Bit Defines */
/* ----------------------------- */
#define ADC_DEC0_DEC00	0x01	/*  */
#define ADC_DEC0_DEC01	0x02	/*  */ 
#define ADC_DEC0_DEC02	0x04	/*  */ 
#define ADC_DEC0_DEC03	0x08	/*  */
#define ADC_DEC0_DEC04	0x10	/*  */
#define ADC_DEC0_DEC05	0x20	/*  */
#define ADC_DEC0_DEC06	0x40
#define ADC_DEC0_DEC07	0x80


/* ADC DEC1 Register Bit Defines */
/* ----------------------------- */
#define ADC_DEC1_DEC08	0x01	/*  */
#define ADC_DEC1_DEC09	0x02	/*  */ 
#define ADC_DEC1_DEC10	0x04	/*  */ 
#define ADC_DEC1_RESERV	0x08	/* Reserved */
#define ADC_DEC1_SMODE0	0x10	/* Setting Mode Bit 0 */
#define ADC_DEC1_SMODE1	0x20	/* Setting Mode Bit 1 */
#define ADC_DEC1_UB		0x40	/* 0 - Bipolar, 1 - Unipolar */
#define ADC_DEC1_DRDY	0x80	/* Data Ready - Read Only Bit */ 

/* Setting Modes */
#define ADC_SETTING_AUTO	0
#define ADC_SETTING_FAST	ADC_DEC1_SMODE0
#define ADC_SETTING_SINC2	ADC_DEC1_SMODE1
#define ADC_SETTING_SINC3	(ADC_DEC1_SMODE0 | ADC_DEC1_SMODE1)

/* 
	ADC1216 Sampling Freq - This 11 Bit Value is loaded into the ADC1216 Registers 
	DEC1 (3 MSBs) and DEC0 (Lower 8 LSBs).
*/
#define ADC_SAMPLE_FREQ		0x0300 
#define ADC_SAMPLE_FREQ_MSB ((ADC_SAMPLE_FREQ >> 8) & 0x07) 
#define ADC_SAMPLE_FREQ_LSB (ADC_SAMPLE_FREQ & 0x00FF) 


/* these are predefined values for ADC registers */
#define MUX_AINCOM 0x08

/* the setup register */
#define STP_BUF_EN 0x02	/* Enable Buffer                          */
#define STP_REF_HI 0x04	/* 0 - 1.25V, 1 - 2.5V Internal Reference */ 
#define STP_REF_EN 0x08 /* Enable Internal Voltage Reference      */

/* the mode and decimation register */
#define MDEC_UB    0x40




typedef struct cardinfo{
	struct cardinfo* next;
	IOSCANPVT ioscanpvt;
	word isconfigured;
	word oldispresent;
	/*
		used to detect a reinsertion of a carrier card.
		see subroutine ispresent() below.
	*/

	/* the following are arguments to CONFIGURE */
	word cardnum;
	word vmeslotnum;
	word ipslotnum;

	word clocksource;
	word gainval;		/*	the number corresponding to the selected
							gain that is written to the ADC registers */
	word gain;			/*	the gain as selected by the user */
	word clockrateval;	/*	the value written to the Xilinx register */

	word unipolar;
	word aisingle;		/*	3 if aitype is Thermocouple
							2 if aitype is PT-100
							1 if aitype is single ended.
							0 is differential */
	word vref;
	word dacrange1,dacrange2;
	word samples;
	word spacing;
	word trigger;
	word hwInhibEnable;
	word IPmemsize;

	word numchanchip;
	word numchantot;

	longword memperchan;
	volatile longword regbase; /* base address of register address */
	volatile longword membase; /* RAM base address */

	/* these are copies of registers for hotswapping */
	word csr;
	word sampreg;
	word intvecnum;
	word clockreg;
	word xilinxacr;

	/* these are set to values of the ADC registers as needed */
	byte adcsetup;
	byte adcmux;
	byte adcacr;
	byte adcidac1;
	byte adcodac;
	byte adcdec0;
	byte adcmdec1;

	/* Thermocouple - Raw Calibration Values */
	longword adcAzero;	/* ADC A - Zero Raw Calibration */
	longword adcAposfs;	/* ADC A - Pos  Raw Calibration */
	longword adcAnegfs;	/* ADC A - Neg  Raw Calibration */
	longword adcBzero;	/* ADC B - Zero Raw Calibration */
	longword adcBposfs;	/* ADC B - Pos  Raw Calibration */
	longword adcBnegfs;	/* ADC B - Neg  Raw Calibration */

	/* Thermocouple - Calibration Calculation Values */
	longword adcApsFS;
	longword adcAngFS;
	longword adcBpsFS;
	longword adcBngFS;

	sem_t csem;
}cardinfo;


/*******************GLOBAL VARIABLES***********************/

static char* verstr="$Id: drvHy8403.c,v 1.2 2005/11/24 15:54:40 cross Exp $";
static char* devstr="drvHy8403";
static int  debugai=0;
static int  debugao=0;

static cardinfo* cardlst=NULL;

/* Thermocouple Calibration */
char bCalibrationCount = 2;
unsigned int InterruptCount = 0;

/*
	Array of Values for ADC PGA MUX 


*/

static word PGAMuxSettings[16][4]={

/* Dif   S/E  PT100 Thermo */
	
{ 0x01, 0x08, 0x01, 0x08 },		/* ADC A - Channel 0 */
{ 0x01, 0x08, 0x01, 0x08 },		/* ADC B - Channel 0 */
{ 0x23, 0x18, 0x12, 0x18 },		/* ADC A - Channel 1 */
{ 0x23, 0x18, 0x12, 0x18 },		/* ADC B - Channel 1 */
{ 0x45, 0x28, 0x23, 0x28 },		/* ADC A - Channel 2 */
{ 0x45, 0x28, 0x23, 0x28 },		/* ADC B - Channel 2 */
{ 0x67, 0x38, 0x34, 0x38 },		/* ADC A - Channel 3 */
{ 0x67, 0x38, 0x34, 0x38 },		/* ADC B - Channel 3 */
{ 0x00, 0x48, 0x45, 0x48 },		/* ADC A - Channel 4 */
{ 0x00, 0x48, 0x45, 0x48 },		/* ADC B - Channel 4 */
{ 0x00, 0x58, 0x56, 0x58 },		/* ADC A - Channel 5 */
{ 0x00, 0x58, 0x56, 0x58 },		/* ADC B - Channel 5 */
{ 0x00, 0x68, 0x67, 0x68 },		/* ADC A - Channel 6 */
{ 0x00, 0x68, 0x67, 0x68 },		/* ADC B - Channel 6 */
{ 0x00, 0x78, 0x78, 0x78 },		/* ADC A - Channel 7 */
{ 0x00, 0x78, 0x78, 0x78}}; 	/* ADC B - Channel 7 */


/**********************************************************

    NAME:	HSshutdown()

    Description:

    Function -	Try to shutdown the card gracefully 
				(for hotswap).


    Parameters:

    Inputs:         card = descriptor of the card to be 
					shut down.
    Outputs:        None.
    Return:         

    Notes:
	disable interrupts,
	don't accept any more chars for input or output,
	copy any chars in the read FIFOs to input buffer.

    Programmer(s):  Darrell Nineham

**********************************************************

Revision History
----------------

Date            By      Version     Descripition
----            --      -------     ------------
02-NOV-2004     DAN     1.1.1.3     Intial Version.
    
**********************************************************/
static int HSshutdown(cardinfo *card)
{
    /* declare local variables */
    /* volatile int    regbase = card->regbase; */
    /*int             numchan = card->numchan; */
    int             retval = OK;

	printf("Hot Swap SHUTDOWN Called !\n");

    /* turn off interrupts from this card */

    /* loop through each available channel on the card */
    /*for (ch = 0; ch < numchan; ch++)
    { */
        /* get the channel's properties */

		/* shutdown the card */
    /* } */

    /* time to be gone */
    return retval;

}	/* End of HSshutdown() */


/**********************************************************

    NAME:	HSinit()

    Description:

    Function - Try to restart card gracefully (after hotswap).


    Parameters:

    Inputs:		card = descriptor of the card to be 
				re-intialised.   
    Outputs:    None.  
    Return:         

    Notes:

    Programmer(s):  Darrell Nineham

**********************************************************

Revision History
----------------

Date            By      Version     Descripition
----            --      -------     ------------
02-NOV-2004     DAN     1.1.1.3     Intial Version.
    
**********************************************************/
static int HSinit(cardinfo* card)
{
    /* declare local variables */
    /* word            numchan = card->numchan;
     TY_HYOCTAL_DEV  *dev = card->dev; */

	printf("Hot Swap INIT Called !\n");
    
    /* check if hardware has been initialised */
    /* ----- if  ((res = HWinit(card)) != OK) return res; ---- */

    /* For each configured IP Card that are active */
    /* for (nc = 0; nc < numchan; nc++)
    { */
        /* get this channel's configuration */
        /* ------ dev = &(card->dev[card->inuse[nc]]); ------ */

        /* Configure The Card */
     
        
        /* Wait a bit before writing to next UART */
    /*    taskDelay(10);
    } */
    
    /* time to be gone */
    return OK;

}	/* End of HSinit() */


/**********************************************************

    NAME:	HOTSWAPCB()

    Description:

    Function - Handle a hotswap event.

    Parameters:

    Inputs:			vmeslotnum = the VME slot of the board 
					generating the event,
					hsevent = the type of hotswap event.
    Outputs:        None.        
    Return:         

    Notes:
	This routine will be called by the drvIpac layer when 
	a hotswap event (board removal, reinsertion) occurs.

    Programmer(s):  Darrell Nineham

**********************************************************

Revision History
----------------

Date            By      Version     Descripition
----            --      -------     ------------
02-NOV-2004     DAN     1.1.1.3     Intial Version.
    
**********************************************************/
static int HOTSWAPCB(unsigned short vmeslotnum, HS_event hsevent)
{
    /* declare local variables */
    cardinfo    *card = cardlst;
    int         res;
    int         retval = OK;

    /* loop through each card int the card list */
    for (card = cardlst; card != NULL; card = card->next)
    {
        /* check if this card is on the VME card that raised the event */
        if  (card->vmeslotnum == vmeslotnum)
        {
            /* parse the hotswap event for this card */
            switch (hsevent)
            {
                /* man the lifeboats, she's going down */
                case HS_Extract:
                    card->isconfigured = FALSE;
                    if  ((res = HSshutdown(card)) != OK) retval = res;
                    break;

                /* wilkommen, bienvenue, welcome */
                case HS_Insert:
                    if  ((res = HSinit(card)) != OK) retval = res;
                    card->isconfigured = TRUE;
                    break;
            }
        }
    }

    /* return a compound success/fail state */
    return retval;

}	/* End of HOTSWAPCB() */


/**********************************************************

    NAME:  Hy8403Debug()

    Description:

    Function - A simple accessor routine used to set the
	the global flag	debugai and debugao at runtime.


    Parameters:

    Inputs:     int ai - Passed Debug Analog Input. 
				int ao - Passed Debug Analog Output. 
    
	Outputs:	debugai -	Whether to display debug of 
							Analog Input.
				debugao -	Whether to display debug of 
							Analog Output.
    Return:		None.         

    Notes:

    Programmer(s):  Darrell Nineham
                    Walter Scott            
                    Steve Hunt

**********************************************************

Revision History
----------------

Date            By      Version     Descripition
----            --      -------     ------------
19-OCT-2004     DAN     1.1.1.3     Converted to Hytec
                                    Coding Standard.
    
**********************************************************/
void Hy8403Debug(int ai,int ao)
{
	debugai=ai;
	debugao=ao;
}


/**********************************************************

    NAME:  scowait()

    Description:

    Function - Simply Loop based delay routine

    Parameters:

    Inputs:         None.
    Outputs:        None.
    Return:         None.

    Notes:

    Programmer(s):  Darrell Nineham
                    Walter Scott            

**********************************************************

Revision History
----------------

Date            By      Version     Descripition
----            --      -------     ------------
19-OCT-2004     DAN     1.1.1.3     Converted to Hytec
                                    Coding Standard.
    
**********************************************************/
static void scowait()
{
    int i,j;
    for (i=0; i<20; i++)
    {
        for (j=0; j<100000; j++);
    }
}


/**********************************************************

    NAME:  rdadc()

    Description:

    Function - Read the Requested ADC Register. 

    Parameters:

    Inputs:         rr - ADC's base address.

					adcsel - which ADC to Read...
					0x0000 = ADC A
					0x8000 = ADC B

					adcreg - which ADC Register to read.

    Outputs:        None.

    Return:			The contents of the requested register.        

    Notes:

    Programmer(s):  Darrell Nineham
                    Walter Scott            
                    Steve Hunt

**********************************************************

Revision History
----------------

Date            By      Version     Descripition
----            --      -------     ------------
19-OCT-2004     DAN     1.1.1.3     1. Ensure ADC dis-armed
									before reading.
									2. Converted to Hytec
                                    Coding Standard.
    
**********************************************************/
static byte rdadc(longword rr,word adcsel,word adcreg)
{
	volatile word*		csrptr=(word*)rr;
	volatile longword	reg=rr;
	word				ww;
	word				op;
	word				wValueCSR;

	/* begin */

	/* Remember the present CSR Value */
	wValueCSR = *csrptr;

	/* Disarm ADC - the ADC Registers can only be read then */ 
	*csrptr &= ~CSR_ARM;

	/* 
		Prepare ADC Operation Code for a single byte read 
		The OP Code is of the format 0xX1R2
		where...
		X - 0x0000=ADC A or 0x8000=ADC B
		R - Which ADC Register 0-F. 
	*/	
	op = adcsel | CON_RREG8 | (adcreg <<4);

	/* Wait for Ready */
	while ((*csrptr & CSR_RDY)==0);scowait();

	/* Write Generated OP Code ADC Control Register */
	*((word*)(reg + REG_ADCCON))=op;

	/* Wait for Ready */
	while ((*csrptr & CSR_RDY)==0);scowait();

	/* Read for ADC Data Register */
	ww=*((word*)(reg + REG_ADCDAT));
	
	/* Restore the entry CSR Value */
	*csrptr = wValueCSR;
	
	/* ADC Data in top 8 bits, so shift it and return it */
	return (byte)(ww>>8);
}


/**********************************************************

    NAME:  wradc()

    Description:

    Function - Update the specified ADC Register with the
    passed value.

    Parameters:

    Inputs:         rr - ADC's base address.

					adcsel - which ADC to Read...
					0x0000 = ADC A
					0x8000 = ADC B

					adcreg - which ADC Register to read.

                    val - Value to update ADC Register with.

    Outputs:        None.
    Return:         None.

    Notes:

    Programmer(s):  Darrell Nineham
                    Walter Scott            
                    Steve Hunt

**********************************************************

Revision History
----------------

Date            By      Version     Descripition
----            --      -------     ------------
19-OCT-2004     DAN     1.1.1.3     Converted to Hytec
                                    Coding Standard.
    
**********************************************************/
static void wradc(longword rr,word adcsel,word adcreg,byte val)
{
	volatile longword reg=rr;
	word ww;
  
	/* begin */
	ww=val;ww=ww<<8;
	while (!(*((word*)(reg+REG_CSR)) & CSR_RDY));scowait();
	*((word*)(reg+REG_ADCDAT))=ww;
	*((word*)(reg+REG_ADCCON))=adcsel | CON_WREG8 | (adcreg <<4);
	while (!(*((word*)(reg+REG_CSR)) & CSR_RDY));scowait();
}


/**********************************************************

    NAME:  dumpadc()

    Description:

    Function - Display both sets of ADC registers for the
    requested card.

    Parameters:

    Inputs:         card - which card's ADC registers to
                    display.
    Outputs:        None.       
    Return:         None.         

    Notes:

    Programmer(s):  Darrell Nineham
                    Walter Scott            
                    Steve Hunt

**********************************************************

Revision History
----------------

Date            By      Version     Descripition
----            --      -------     ------------
07-SEP-2005     DAN     1.7         Added ADC Title.
    
19-OCT-2004     DAN     1.1.1.3     Converted to Hytec
                                    Coding Standard.
    
**********************************************************/

/* Set up a Bit Mask Array for ADC A and B */
static word adctab[2]={CON_ADCA,CON_ADCB};

static void dumpadc(cardinfo* card)
{
	word adcsel;
	volatile longword reg=card->regbase;
	int i,ii;
  
	printf("\n");
    
    /* Read Both ADC 0=ADC1216-1(A) and  1=ADC1216-2(B) */
	for (ii=0;ii<2;ii++)
	{
		printf("ADC %c\n",'A'+ii);
		printf("------\n\n");
        
		/* Set up adcsel with ADC Bit Mask */
		adcsel=adctab[ii];

		/* For all ADC Registers, Read and Display Them */
		for (i=0;i<NUMADCREG;i++)
		{
			printf("ADC REG %2d: 0x%2x\n",i,rdadc(reg,adcsel,i));
		}
		printf("\n");
	}
}


/**********************************************************

    NAME:  dumpxilinx()

    Description:

    Function - Displays the hardware registers of the 
    requested IP Card, the display is made up of... 
    1. The Xilinx Registers
	2. The ADC Mux Setup Registers
	3. The Actual Present ADC Values
    4. Both ADCs Setup Registers  

    Parameters:

    Inputs:         card - which card's registers to
                    display.
    Outputs:        None.     
    Return:         None.

    Notes:

    Programmer(s):  Darrell Nineham
                    Walter Scott            
                    Steve Hunt

**********************************************************

Revision History
----------------

Date            By      Version     Descripition
----            --      -------     ------------
19-OCT-2004     DAN     1.1.1.3     1. Copes with Extra 
                                    Registers in Xilinx.
                                    2. Now Displays ADC
                                    Registers Aswell.
                                    3. Converted to Hytec
                                    Coding Standard.
    
**********************************************************/
static void dumpxilinx(cardinfo* card)
{
	volatile longword reg=card->regbase;
	int i;
	word hi,lo;
	longword ww;
  
	/* begin */

    /* Display the Xilinx Registers */
	for (i=0; i<9; i++)
	{
		printf("XILINX REG    %2d: 0x%04x\n",i*2,*((word*)(reg+i*2)));
	}
  
	/* Display the ADC Mux Setup Registers */
    reg = card->regbase + REG_PGAMUX;
	for (i=0; i<16; i++)
	{
		printf("XILINX MUXREG %2d: 0x%04x\n",i*2,*((word*)(reg+i*2)));
	}

	/* Display the Actual Present ADC Values */
    reg = card->regbase + REG_ADCBASE;
	for (i=0;i<16;i++)
	{
		lo=*((word*)reg);reg+=2;
		hi=*((word*)reg);reg+=2;
		ww=(hi<<16)+lo;
		printf("XILINX DATA   %2d: 0x%08x\n",i,ww);
	}

    /* Display both ADCs Setup Registers */  
	dumpadc(card);
}


/**********************************************************

    NAME:  scodump()

    Description:

    Function - Accessor Function to allow user to easily
    display any particular cards 

    Parameters:

    Inputs:         cardnum - which IP card to display by
    number, which it is identified/registered by. 
    Outputs:        None.
    Return:         None.

    Notes:  Passed Parameters changed from W.Scott's orginal
    to ease use.  Orginal kept commented out for backward
    compatibility.

    Programmer(s):  Darrell Nineham
                    Walter Scott            
                    Steve Hunt

**********************************************************

Revision History
----------------

Date            By      Version     Descripition
----            --      -------     ------------
19-OCT-2004     DAN     1.1.1.3     1. Changed passed 
                                    parameter.
                                    2. Converted to Hytec
                                    Coding Standard.
    
**********************************************************/
void scodump(int cardnum)
{
	cardinfo* card=cardlst;
	
	/* begin */
	while(card!=NULL && card->cardnum!=cardnum)card=card->next;
	
	if (card==NULL)
	{
		printf("scodump: no cardnumber %d found\n",cardnum);
		return;
	}

	dumpxilinx(card);
}

  
/* Orginal Scotty Code
void scodump(int i,int val)
{
	cardinfo* card=cardlst;
	volatile longword dstptr=card->regbase;
	word w=val;
  
	if (i>=0)
	{
		printf("writing 0x%04x to 0x%08x\n",w,dstptr+i);
		*((word*)(dstptr+i))=w;
	}
	if (val>=0)
    {
		dumpxilinx(card);

	}
}
*/


/**********************************************************

    NAME:  scobla()

    Description:

    Function - 

    Parameters:

    Inputs:         None.
    Outputs:        
    Return:         

    Notes:

    Programmer(s):  Darrell Nineham
                    Walter Scott            
                    Steve Hunt

**********************************************************

Revision History
----------------

Date            By      Version     Descripition
----            --      -------     ------------
19-OCT-2004     DAN     1.1.1.3     Converted to Hytec
                                    Coding Standard.
    
**********************************************************/
void scobla()
{
	/*int restart;*/
	cardinfo* card=cardlst;
	
	/* begin */
	/*
	restart=isarmed(card);
	if (restart)
	{
		printf("stopping ADC scanning\n");
		armadc(card,0);
	}
	dumpadc(card);
	*/

	dumpxilinx(card);
	
	/*
    if (restart)
	{
		printf("restarting scanning\n");
		armadc(card,1);
    }
	*/
}


/**********************************************************

    NAME:  isarmed()

    Description:

    Function - 

    Parameters:

    Inputs:         None.
    Outputs:        
    Return:         

    Notes:

    Programmer(s):  Darrell Nineham
                    Walter Scott            
                    Steve Hunt

**********************************************************

Revision History
----------------

Date            By      Version     Descripition
----            --      -------     ------------
19-OCT-2004     DAN     1.1.1.3     Converted to Hytec
                                    Coding Standard.
    
**********************************************************/
static int isarmed(cardinfo* card)
{
	volatile word* csr=(word*)(card->regbase+REG_CSR);
	int res;
	if (sem_wait(&(card->csem))!=OK)
	{
		printf("ERRORRORR ISARMED SEM_WAIT!\n");
	}
	res=((*csr & CSR_ARM)!=0);
	if (sem_post(&(card->csem))!=OK)
	{
		printf("ERRORRORR ISARMED SEM_POST!\n");
	}
	return res;
}


/**********************************************************

    NAME:  armadc()

    Description:

    Function - Called to Arm and Disarm ADC on specific
    card.

    Parameters:

    Inputs:         card    - which card to Arm/Disarm.
                    int arm - 1=Arm ADC  0=Disarm ADC. 
    Outputs:        None.
    Return:         Nothing.

    Notes:

    Programmer(s):  Darrell Nineham
                    Walter Scott            
                    Steve Hunt

**********************************************************

Revision History
----------------

Date            By      Version     Descripition
----            --      -------     ------------
19-OCT-2004     DAN     1.1.1.3     Converted to Hytec
                                    Coding Standard.
    
**********************************************************/
static void armadc(cardinfo* card,int arm)
{
	volatile word* csr=(word*)(card->regbase+REG_CSR);
	int debug=(debugai || debugao);
  
	/* begin */
	if (debug)printf("ARMADC %d\n",arm);
	if (sem_wait(&(card->csem))!=OK)
	{
		printf("ERRORRORR ARMADC SEM_WAIT!\n");
		return;
	}
  
	if (arm == 1)
	{
		*csr |= CSR_ARM;
	}
	else
	{
		/*	
            if we disarm, wait until the
			last instruction has been completed
        */
		*csr &= (~CSR_ARM);
		while (!(*csr & CSR_RDY));scowait();
	}
  
	if (sem_post(&(card->csem))!=OK)
	{
		printf("ERRORRORR ARMADC SEM_POST!\n");
		return;
	}

	if (debug)printf("read back 0x%04x\n",*csr);
}


/**********************************************************

    NAME:			initadc()

    Description:

    Function - Called to Initialise Both ADCs

    Parameters:

    Inputs:         card - which card data structure to 
    Outputs:        None.
    Return:         None.

    Notes:

    Programmer(s):  Darrell Nineham
                    Walter Scott            
                    Steve Hunt

**********************************************************

Revision History
----------------

Date            By      Version     Descripition
----            --      -------     ------------
19-OCT-2004     DAN     1.1.1.3     Converted to Hytec
                                    Coding Standard.
    
**********************************************************/
static void initadc(cardinfo* card)
{
	volatile longword reg=card->regbase;
	word unipolar=card->unipolar;
	volatile word* rr;
	word adcsel;
	byte bb;
	word ii;
  
	/* begin */

	/* zero the mux registers of the xilinx */
	rr=(word*)(reg+REG_PGAMUX);
	for (ii=0;ii<16;ii++)*rr++=0;


	/* perform a reset of the ADC's */
	while (!(*((word*)(reg+REG_CSR)) & CSR_RDY));scowait();
	*(word*)(reg+REG_ADCCON)=CON_ADCA | CON_RESET;
	while (!(*((word*)(reg+REG_CSR)) & CSR_RDY));scowait();
	*(word*)(reg+REG_ADCCON)=CON_ADCB | CON_RESET;
	while (!(*((word*)(reg+REG_CSR)) & CSR_RDY));scowait();

	/* Step Through Both ADCs */
	for (ii=0;ii<2;ii++)
	{
		/* Load adcsel with the bit mask for each ADC */
		adcsel=adctab[ii];

		/* now perform a calibration */
		while (!(*((word*)(reg+REG_CSR)) & CSR_RDY));scowait();
		*((word*)(reg+REG_ADCCON))=adcsel | CON_CALOG;
		while (!(*((word*)(reg+REG_CSR)) & CSR_RDY));scowait();

		/* set up the ADC/DAC registers  */
		bb=card->adcsetup=(STP_BUF_EN | STP_REF_HI |STP_REF_EN);
		wradc(reg,adcsel,ADC_SETUP,bb);

		/* DAC setup -- the range */
		bb=(card->dacrange1<<3)+card->gainval;
		wradc(reg,adcsel,ADC_ACR,bb);

		/* set the IDAC value to a default */
		wradc(reg,adcsel,ADC_IDAC1,0x12);

		/*	
			set the uni/bipolar mode and the decimation rate
			THIS WILL HAVE TO BE CHANGED DEPENDING ON XILINX 
			CLOCK RATE THESE VALUE ARE CHOSEN TO BE SUITABLE 
			FOR XILINX CLOCKRATE OF 10Hz.
		*/
		bb=0xc0;
		wradc(reg,adcsel,ADC_DEC0,bb);

		bb=0x0;
		if (unipolar)bb|=(byte)MDEC_UB;
		wradc(reg,adcsel,ADC_DEC1,bb);
	}

	/* Wait until the Xilinx Ready */
	while (!(*((word*)(reg+REG_CSR)) & CSR_RDY));scowait();
}


/**********************************************************

    NAME:  checkprom()

    Description:

    Function - 
	print out the contents of the prom.
	pr is the first address of the prom

    Parameters:

    Inputs:         None.
    Outputs:        
    Return:         

    Notes:

    Programmer(s):  Darrell Nineham
                    Walter Scott            
                    Steve Hunt

**********************************************************

Revision History
----------------

Date            By      Version     Descripition
----            --      -------     ------------
19-OCT-2004     DAN     1.1.1.3     Converted to Hytec
                                    Coding Standard.
    
**********************************************************/

/* the value expected in the board's PROM for model number */
#define PROM_MODEL 0x8403
#define PROM_OFFS  0x80

static int checkprom(int pr,int expmodel,int verbose)
{
	#define PR_ID    0x06
	#define PR_MODN  0x0a
	#define PR_REVN  0x0c
	#define PR_DRID1 0x10
	#define PR_DRID2 0x12
	#define PR_FLAGS 0x14
	#define PR_BYTES 0x16
	#define PR_SERN  0x1a
	#define HYTECID  0x00800300
    
    /* declare local variables */
	char*   expstr="VITA4 ";
	char*   hytecstr="  (HyTec Electronics Ltd., Reading, UK)";
	char    lstr[7];
	int     manid, ishytec, ismodel, strok;
	word    modelnum;
  
	/* begin */
	if (verbose) printf("PROM CONTENTS AT: %X\n",pr);

    /* get the PROM header and compare */
    memcpy(lstr, (char*)pr, 6); lstr[6] = 0;
    strok = (strcmp(lstr, expstr) == 0);
    if (verbose) printf("PROM header: '%s'\n", lstr);

    /* get the manufacturer's ID and compare */
    manid = *((int*)(pr + PR_ID));
    ishytec = (manid == HYTECID);

    /* get the device's model number and compare */
    modelnum = *((word*)(pr + PR_MODN));
	ismodel=(modelnum==expmodel);
	
    /* check if verbose (or debug) output requested */
    if  (verbose)
    {
        /* output a summary of the PROM contents */
        printf("PROM CONTENTS AT: 0x%04x\n", pr);
        printf("PROM manufacturer ID: 0x%08x, %s\n\n", manid, ((ishytec) ? hytecstr : "(unknown)"));
        printf("PROM model number: 0x%04x, rev. 0x%04x\n", modelnum, *((short*)(pr + PR_REVN)));
        printf("PROM serial number: %u\n", *((short*)(pr + PR_SERN)));
        printf("PROM driver IDs: 0x%04x, 0x%04x\n", *((short*)(pr + PR_DRID1)), *((short*)(pr + PR_DRID2)));
        printf("PROM flags: 0x%04x\n", *((short*)(pr + PR_FLAGS)));
        printf("PROM bytes used: 0x%04x (%d)\n\n", *((short*)(pr + PR_BYTES)), *((short*)(pr + PR_BYTES)));
    }

    if  (!strok)    printf("PROM ! INVALID PROM HEADER: EXPECTED '%s'\n", expstr);
    if  (!ishytec)  printf("PROM ! INVALID MANUFACTURER ID: EXPECTED 0x%08x, %s\n", HYTECID, hytecstr);
    if  (!ismodel)  printf("PROM ! INVALID MODEL NUMBER: EXPECTED 0x%04x\n", expmodel);

    /* return a compond success/fail code */
    return (strok && ishytec && ismodel);

    /* if (verbose)
	{
		printf("PROM manufacturer ID: 0x%08X",manid);
		if (ishytec)printf(hytecstr);

		printf("\nPROM model #: 0x%04hx, rev. 0x%04hx, serial # %hu\n",
		modelnum,
		*((short int*)(pr+PR_REVN)),
		*((short int*)(pr+PR_SERN)));
    
		printf("PROM driver ids: 0x%04hx, 0x%04hx\n",
		*((short int*)(pr+PR_DRID1)),
		*((short int*)(pr+PR_DRID2)));
    
		printf("PROM flags: 0x%04hx\nPROM number of bytes used: 0x%04hx (%hd)\n\n",
		*((short int*)(pr+PR_FLAGS)),
		*((short int*)(pr+PR_BYTES)),
		*((short int*)(pr+PR_BYTES)));
	}
  
	return (strok && ishytec && ismodel);
	return (strok && ishytec); */
}


/**********************************************************

    NAME:  dumpcard()

    Description:

    Function - 

    Parameters:

    Inputs:         ci - pointer to card structure
    Outputs:        None.
    Return:         OK	- Everything OK
					ERR - Any Write Fails

    Notes:

    Programmer(s):  Darrell Nineham
                    Walter Scott            
                    Steve Hunt

**********************************************************

Revision History
----------------

Date            By      Version     Descripition
----            --      -------     ------------
19-OCT-2004     DAN     1.1.1.3     Converted to Hytec
                                    Coding Standard.
    
**********************************************************/
static int dumpcard(cardinfo* ci)
{
	volatile longword dstptr=ci->regbase;
	volatile longword idst;
	word clkword;
	word locchan;
	word muxword;
  
	/* begin */

	/* If any 'debug' display routine name */
	if (debugai || debugao)printf("DUMPCARD!\n");


	/* Copy ci->sampreg to REG_NCONV */
	if (vxMemProbe((char*)(dstptr+REG_NCONV),
					WRITE,sizeof(word),
					(char*)&(ci->sampreg))!=OK)
	{				
		return ERR;
	}

	
	/* 
		Generate value for Clock Rate / Number of Channels
		Remember...
		1. Clockrate is Bits 0-4
		2. Number of Channels is Bits 12-14
	*/	
	clkword=ci->clockrateval+((ci->numchanchip-1)<<12);
  
	/* Update REG_CLKRAT */ 
	if (vxMemProbe((char*)(dstptr+REG_CLKRAT),
					WRITE,sizeof(word),
					(char*)&clkword)!=OK)
	{
		return ERR;
	}	


	/* Copy ci->intvecnum to REG_INTVEC */
	if (vxMemProbe((char*)(dstptr+REG_INTVEC),
					WRITE,sizeof(word),
					(char*)&(ci->intvecnum))!=OK)
	{
		return ERR;
	}

	
	/* Copy ci->xilinxacr to REG_ACR */
	if (vxMemProbe((char*)(dstptr+REG_ACR),
					WRITE,sizeof(word),
					(char*)&(ci->xilinxacr))!=OK)
	{
		return ERR;
	}


	/* set up the XILINX MUX registers */
	/* ------------------------------- */

	/* Set up address of ADCs First PGAMUX Registers */ 
	idst=dstptr+REG_PGAMUX;
	
	/* Work through the number of channels on each ADC */
	for (locchan=0;locchan<ci->numchantot;locchan++)
	{
		/*	Generate MUX Control + Gain...
			Bit 0-3  Negative Channel Select.
			Bit 4-7  Posative Channel Select.
			Bit 8-10 Mapped Gain Value 0-1 1-2 2-4 3-8 etc. */ 
		muxword= (PGAMuxSettings[locchan][ci->aisingle])|(ci->gainval<<8);
				
	    /* Load Channel with PGA Mux Settings */
		while (!(*((word*)(ci->regbase + REG_CSR)) & CSR_RDY));scowait();
		*(word*)(idst) = muxword;
                
		/* Move Address to next Channel */
		idst+=2;

		/* If debug level suitably high display debug */
		if (debugai>1)
		{
			printf("MUX 0x%08x  0x%04x 0x%04x\n",
					idst,muxword,*(word*)idst);
		}
	}

	if (sem_wait(&(ci->csem))!=OK)
	{
		printf("ERRORRORR DUMPCARD SEM_WAIT!\n");
		return ERR;
	}  
  
	/*	
		write CSR last to ensure that we have the correct 
		setup before we arm and interrupts are enabled 
	*/
	if (vxMemProbe((char*)(dstptr+REG_CSR),
					WRITE,sizeof(word),
					(char*)&(ci->csr))!=OK)
	{
		return ERR;
	}

	if (sem_post(&(ci->csem))!=OK)
	{
		printf("ERRORRORR DUMPCARD SEM_POST!\n");
		return ERR;
	}

	return OK;
}


/**********************************************************

    NAME:  ispresent()

    Description:

    Function - 

    Parameters:

    Inputs:         None.
    Outputs:        
    Return:	TRUE if the card is present.
			If it has been reinserted since the last
			query, reinitialise the hardware

    Notes:

    Programmer(s):  Darrell Nineham
                    Walter Scott            
                    Steve Hunt

**********************************************************

Revision History
----------------

Date            By      Version     Descripition
----            --      -------     ------------
19-OCT-2004     DAN     1.1.1.3     Converted to Hytec
                                    Coding Standard.
    
**********************************************************/
static int ispresent(cardinfo* p)
{
	int curstate;
	
	/* begin */
	curstate=ipmCarrierIsPresent(p->vmeslotnum);
	
	/* check for reinsertion */
	if (curstate>0 && p->oldispresent==0)
	{
		initadc(p);
		curstate=(dumpcard(p)==OK);
		if (curstate)armadc(p,1);
	}
	p->oldispresent=curstate;
	return curstate;
}


/**********************************************************

    NAME:  get8403ioscanpvt()

    Description:

    Function - 

    Parameters:

    Inputs:         None.
    Outputs:        
    Return:         

    Notes:

    Programmer(s):  Darrell Nineham
                    Walter Scott            
                    Steve Hunt

**********************************************************

Revision History
----------------

Date            By      Version     Descripition
----            --      -------     ------------
19-OCT-2004     DAN     1.1.1.3     Converted to Hytec
                                    Coding Standard.
    
**********************************************************/
int get8403ioscanpvt(word cardnum, IOSCANPVT *scanpvt)
{
	cardinfo* card=cardlst;
	/* begin */
	while (card!=NULL && card->cardnum!=cardnum)card=card->next;
	if (card==NULL)
	{
		printf("get8403ioscanpvt: cardnum %d not found",cardnum);
		return ERR;
	}
	*scanpvt=card->ioscanpvt;
	return OK;
}


/**********************************************************

    NAME:			intconfig()

    Description:

    Function -

    Parameters:

    Inputs:
                
    int cardnum     -   card number as used in INP fields
	int vmeslotnum  -   vmeslot of the carrier board
	int ipslotnum   -   IP slot number
	int vectornum   -   interrupt vector number. 0=disabled
	int aitype      -   0=8 differential,
                        1=16 single ended,
                        2=14 PT100
                        3=13 Thermocouples
	int IPmemsize   -   1=1MB memory, 2=2MB memory
	int clocksource -   0=internal, 1=external
	int clockrate   -   Xilinx scanning freq(Htz)
				        legal values - 1,2,5,10,20,50,100 & 200
	int gain        -   gain for all ADC inputs
				        valid values - 1,2,4,8,16,32,64 or 128
	int Vref        -   Internal ADC ref volt 1=1.25V, 2=2.5V
    int unipolar    -   0=bipolar ADC input, 1=unipolarADC input
    int hwInhibEnable   -   1=front panel inhibit
	int samples		-	number of samples to average for ai 
	                     1 means no averaging -use registers
	                     0 means use 100 samples and 10 spacing 
	int spacing		-	spacing between samples to average for ai 
    int trigger		-   0 continuous,
	                    >0 trigger mode, convert trigger values
	                    when triggered by HW or SW.
	                    (trigger overrides samples in this case). 
    int DACrange1,
	int DACrange2	-	Range Select for DACS.
	                    See the Hy8403 user manual for an 
						explanation of these values.
	                    =0: off
	                    =1: range 1
	                    =2: range 2
	                    =3: range 3 


	Outputs:
                

	Return:     
				OK	-	None.
				ERR	-	

    Notes:

    Programmer(s):  Steve Hunt
                    Walter Scott            

**********************************************************

Revision History
----------------

Date            By      Version     Descripition
----            --      -------     ------------
19-OCT-2004     DAN     1.1.1.3     Converted to Hytec
                                    Coding Standard.
    
**********************************************************/
static int intconfig(   int cardnum,
	                    int vmeslotnum,
	                    int ipslotnum,
	                    int vectornum,
	                    int aitype,
	                    int IPmemsize,
	                    int clocksource,
	                    int clockrate,
	                    int gain,
	                    int Vref,
	                    int unipolar,
	                    int hwInhibEnable,
	                    int samples,
	                    int spacing,
	                    int trigger,
	                    int DACrange1,
	                    int DACrange2)
{
    #define CLKTAB 8
    cardinfo* newcard;
    int gaintab[8]={1,2,4,8,16,32,64,128};
    int clkrate[CLKTAB]={1,2,5,10,20,50,100,200};
    int clkval[CLKTAB]={0,1,2,3,4,7,8,9};
    int gval,i,res;
    word clockrateval;

    /* begin */
    printf("%s\n",verstr);
    printf("CONFIGURE card # %d, carrier slot # %d, ipslot # %d, vectornum %d\n",
																		cardnum,
																		vmeslotnum,
																		ipslotnum,
																		vectornum);
  
    printf("aitype %d, IPmemsize %d,clocksrc %d, clockrate %d, gain %d\n",
																		aitype,
																		IPmemsize,
																		clocksource,
																		clockrate,
																		gain);
  
    printf("Vref %d, unipolar %d, hwInhibEnable %d, samples %d, spacing %d, trigger %d\n",
																		Vref,
																		unipolar,
																		hwInhibEnable,
																		samples,
																		spacing,
																		trigger);
  
    /* Check for a Valid Vector Number */
    if (vectornum < 0 || vectornum > 255)
    {
        printf("ERROR! illegal interrupt vector number %d\n",vectornum);
        printf("must be in range [0..255]\n");
        return ERR;
    }

    /* Check for Valid ADC Input Type  */
    if (aitype < 0 || aitype > 3)
    {
        printf("ERROR! illegal value for aitype %d\n",aitype);
        printf("must be in range 0 to 2\n");
        return ERR;
    }

    /* Check for Valid Memory Size */
    if (IPmemsize != 1 && IPmemsize != 2)
    {
        printf("ERROR! illegal value for IPmemsize %d\n",IPmemsize);
        printf("must be in range 1 or 2\n");
        return ERR;
    }

    /* Check for valid Clock Source */
    if (clocksource != 0 && clocksource != 1)
    {
        printf("ERROR! illegal value for clocksource %d\n",clocksource);
        printf("must be 0 or 1\n");
        return ERR;
    }


    /* 
        Scan the Clockrate Table(clkrate) to find the passed clockrate, the
        index is the Clock Rate code for the Xilinx REG_CLKRAT and is stored
        in clockrateval. 
    */
    i=0;
    while (i<CLKTAB && clkrate[i] != clockrate)i++;
    
    /* If end of Clockrate Table is passed then a invalid one has been requested */
    if (i==CLKTAB)
    {
        printf("ERROR illegal value for clockrate %d\n",clockrate);
        printf("must be one of:\n");
        for(i=0;i<CLKTAB;i++)printf("%d\n",clkrate[i]);
        return ERR;
    }
    clockrateval=clkval[i];


    /* 
        Scan the Gain Table(gaintab) to find the passed gain, the index
        is the gain code for the ADC and is stored in gainval via gval. 
    */
    gval=0;
    while(gval<8 && gaintab[gval]!=gain)gval++;
    
    /* If end of Gain Table is passed then a invalid gain has been requested */
    if (gval==8)
    {
        printf("ERROR illegal value for gain %d\n",gain);
        printf("must be 1, 2, 4, 8, 16, 32, 64 or 128\n");
        return ERR;
    }


    /* Check for valid Clock Source */
    if (unipolar != 0 && unipolar != 1)
    {
        printf("ERROR illegal value for unipolar %d\n",unipolar);
        printf("must be 0 or 1\n");
        return ERR;
    }

    if (Vref != 1 && Vref != 2)
    {
        printf("ERROR illegal value for Vref %d\n",Vref);
        printf("must be 1 (for 1.25V) or 2 (for 2.5V)\n");
        return ERR;
    }

    if (hwInhibEnable != 0 && hwInhibEnable != 1)
    {
        printf("ERROR! illegal value for hwInhibEnable %d\n",hwInhibEnable);
        printf("must be 0 or 1\n");
        return ERR;
    }
  
    if (samples < 0)
    {
        printf("ERROR! illegal value for samples %d !\n",samples);
        printf("must be >=0\n");
        return ERR;
    }
  
    if (samples != 0 && spacing <= 0)
    {
        printf("ERROR! illegal value for spacing %d !\n",spacing);
        printf("must be >0 if samples>0\n");
        return ERR;
    }

    if (DACrange1 < 0 || DACrange1 > 3)
    {
        printf("ERROR! illegal value for DACrange1 %d !\n",DACrange1);
        printf("must be 0 <= DACrange1 <4\n");
        return ERR;
    }
    
    if (DACrange2 < 0 || DACrange2 > 3)
    {
        printf("ERROR! illegal value for DACrange2 %d !\n",DACrange2);
        printf("must be 0 <= DACrange2 <4\n");
        return ERR;
    }

    newcard=cardlst;
    while (newcard !=NULL)
    {
        if (newcard->vmeslotnum==vmeslotnum &&
	        newcard->ipslotnum==ipslotnum)
        {
            printf("ERROR! card in vmeslot %d and ipslot %d\n",
	            vmeslotnum,ipslotnum);
            printf("already configured as card number %d\n",newcard->cardnum);
            return ERR;
        }
        
        if (newcard->cardnum==cardnum)
        {
            printf("ERROR! card number %d already configured ",cardnum);
            printf("(vmeslot %d, ipslot %d)\n",
	        newcard->vmeslotnum,newcard->ipslotnum);
        }
        newcard=newcard->next;
    }
  
    newcard=calloc(1, sizeof(cardinfo) );
    if (newcard == NULL)
    {
        logMsg ("%s: Memory allocation error\n", (int)devstr,0,0,0,0,0 );
        return ERR;
    }

    newcard->next=cardlst;cardlst=newcard;
    newcard->cardnum=cardnum;
    newcard->vmeslotnum=vmeslotnum;
    newcard->ipslotnum=ipslotnum;
    newcard->isconfigured=FALSE;
    newcard->oldispresent=TRUE;
    newcard->intvecnum=vectornum;
    newcard->hwInhibEnable=hwInhibEnable;
    newcard->IPmemsize=IPmemsize;
    newcard->aisingle=aitype;

    
    if (newcard->aisingle == 3)			/* Used to read Thermocouples */
    {
        newcard->numchanchip =  8;
        newcard->numchantot  = 16;
    }
    else if (newcard->aisingle == 2)	/* Used to read PT-100 */
    {
        newcard->numchanchip =  8;
        newcard->numchantot  = 16;
    }
    else if (newcard->aisingle == 1)	/* Single Ended ADC reading */
    {
        newcard->numchanchip =  8;
        newcard->numchantot  = 16;
    }
    else								/* Differential ADC reading */
    {
        newcard->numchanchip =  4;
        newcard->numchantot  =  8;
    }
    
    
    newcard->gainval=gval;
    newcard->gain=gain;
    newcard->unipolar=unipolar;
    newcard->vref=Vref;
    newcard->dacrange1=DACrange1;
    newcard->dacrange2=DACrange2;

    /* ******** Darrell Nineham - Additional Fix May be Needed here ********* */
    if (IPmemsize==1)
    {
        /* 1MB Size */
        if (aitype)newcard->memperchan=MEM_SINGLE_1MB;
        else       newcard->memperchan=MEM_DIFF_1MB;
    }
    else
    {
        /* 2MB Size */
        if (aitype)newcard->memperchan=MEM_SINGLE_2MB;
        else       newcard->memperchan=MEM_DIFF_2MB;
    }
    newcard->clocksource=clocksource;
    newcard->clockrateval=clockrateval;

    if(samples==0)
    {
        samples=100;spacing=10;
        printf("samples==0: setting samples=%d, spacing=%d\n",
	            samples,spacing);
    }

    newcard->samples=samples;
    newcard->spacing=spacing;
    newcard->trigger=trigger;
    
    /* ******* Darrell HotSwap Fix ******** */
    /* attempt to make carrier hotswappable */
    if  ((res = ipmInstallHSCallBack(vmeslotnum, &HOTSWAPCB)) != OK)
    {
        /* callback failed, discard card structure and abort */
        free(newcard);
        return res;
    }
                                        

    return OK;
}


/**********************************************************

    NAME:  Hy8403SYSOCAL()

    Description:

    Function - 
	calibration routines. These are globally defined
	routines which can be called from the command line.

	system offset calibration.
	The inputs should be set to 0V differential and both ADS1216's
	will compute the OCR register to compensate for offset errors.
	This value is printed to screen before and after calibration


    Parameters:

    Inputs:         None.
    Outputs:        
    Return:         

    Notes:

    Programmer(s):  Darrell Nineham
                    Walter Scott            
                    Steve Hunt

**********************************************************

Revision History
----------------

Date            By      Version     Descripition
----            --      -------     ------------
19-OCT-2004     DAN     1.1.1.3     Converted to Hytec
                                    Coding Standard.
    
**********************************************************/
int Hy8403SYSOCAL(int cardnum)
{
	cardinfo* card=cardlst;

	/* begin */
	while(card!=NULL && card->cardnum!=cardnum)card=card->next;
	if (card==NULL)
	{
		printf("cardnum %d not found!\n",cardnum);
		return  S_drv_noDevice;
	}
	
	/*
	a) read the values from the chips.
    b) write cal opcode.
    c) wait
    d) read and print values 
	*/

	return OK;
}


/**********************************************************

    NAME:  Hy8403SYSGCAL()

    Description:

    Function - system gain calibration.
	The inputs should be set to the reference voltage and both 
	ADS1216's will compute the FSR value for gain errors.
	The values are printed to screen before and after calibration.

    Parameters:

    Inputs:         None.
    Outputs:        
    Return:         

    Notes:

    Programmer(s):  Darrell Nineham
                    Walter Scott            
                    Steve Hunt

**********************************************************

Revision History
----------------

Date            By      Version     Descripition
----            --      -------     ------------
19-OCT-2004     DAN     1.1.1.3     Converted to Hytec
                                    Coding Standard.
    
**********************************************************/
int Hy8403SYSGCAL(int cardnum)
{
	cardinfo* card=cardlst;

	/* begin */
	while(card!=NULL && card->cardnum!=cardnum)card=card->next;
	if (card==NULL)
	{
		printf("cardnum %d not found!\n",cardnum);
		return  S_drv_noDevice;
	}

	return OK;
}


/**********************************************************

    NAME:  hy8403_isr()

    Description:

    Function - 

    Parameters:

    Inputs:         None.
    Outputs:        
    Return:         

    Notes:

    Programmer(s):  Darrell Nineham
                    Walter Scott            
                    Steve Hunt

**********************************************************

Revision History
----------------

Date            By      Version     Descripition
----            --      -------     ------------
19-OCT-2004     DAN     1.1.1.3     Converted to Hytec
                                    Coding Standard.
    
**********************************************************/
static void hy8403_isr(int cardnum)
{
	cardinfo* card=cardlst;
	
	/* begin */
	while(card!=NULL && card->cardnum!=cardnum)card=card->next;
	
	if (card==NULL)
	{
		printf("isr: no cardnumber %d found\n",cardnum);
		return;
	}
  
	/* Clear interrupt */
    
    /* re-enable interrupts */
	scanIoRequest(card->ioscanpvt);
}


/**********************************************************

    NAME:  IO_REPORT()

    Description:

    Function - 

    Parameters:

    Inputs:         None.
    Outputs:        
    Return:         

    Notes:

    Programmer(s):  Darrell Nineham
                    Walter Scott            
                    Steve Hunt

**********************************************************

Revision History
----------------

Date            By      Version     Descripition
----            --      -------     ------------
19-OCT-2004     DAN     1.1.1.3     Converted to Hytec
                                    Coding Standard.
  
**********************************************************/
long IO_REPORT()
{
	cardinfo* card=cardlst;
	volatile longword base;
	
	/* begin */
	printf("%s\n",verstr);
  
	while(card!=NULL)
	{
		base=card->regbase+0x80;
		printf("Hytec8403: card=%X VMEslot=%X IPslot=%X Id=%X vers=%X ser=%d\n",
		card->cardnum,
		card->vmeslotnum,
		card->ipslotnum,
		*((word*)(base+PR_ID)),
		*((word*)(base+PR_REVN)),
		*((word*)(base+PR_SERN)));

		card=card->next;
	}
	return OK;
}


/**********************************************************

    NAME:  DRIVER_INIT()

    Description:

    Function - Driver entry point.
	Loop through the cardlst and initialise all devices in
	the list. The list has been built up by the CONFIGURE 
	routine.

    Parameters:

    Inputs:         None.
    Outputs:        
    Return:         

    Notes:

    Programmer(s):  Darrell Nineham
                    Walter Scott            
                    Steve Hunt

**********************************************************

Revision History
----------------

Date            By      Version     Descripition
----            --      -------     ------------
21-NOV-2004     DAN     1.1.1.2     Enhanced PT-100 H/W
                                    Setup.
    
19-OCT-2004     DAN     1.1.1.1     Converted to Hytec
                                    Coding Standard.
    

**********************************************************/
static long DRIVER_INIT()
{
	cardinfo* card=cardlst;
	int numcards=0;
	int status;
	int isok;
	word rprobe;
	word csr;

	volatile longword idst = 0;
	word locchan;
	word muxword;

	/* For each configured card... */
	while(card!=NULL)
	{
	    /* assume this board can be initialised */
		isok=TRUE;

		/* register the card in the A16 address space */
		card->regbase=(longword)ipmBaseAddr(card->vmeslotnum,
											card->ipslotnum,
											ipac_addrIO);
    
		/* If card can NOT be registered */
		if (card->regbase==NULL)
		{
			/* Log Message */
			logMsg( "%s: Cannot register hy8403 A16 device.",
			(int)devstr,0,0, 0,0,0 );
			/* Indicate Function Exit Required */
			isok=FALSE;
		}

        /* If everything still OK - Check the Register address exsists */
		if (isok)
		{
			/* Verify that something exists at the specified address */
			status=vxMemProbe(	(char *)(card->regbase),
								READ,
								sizeof(word),
								(char *)&rprobe);
			
			/* If Card NOT at Specified address */
			if (status!=OK)
			{
				printf("%s: NO DEVICE: A16 Memory Probe error (status=%d)\n",
				devstr,status);
				printf("card number %d (vmeslot %d, ipslot %d) at address 0x08%x\n",
				card->cardnum,card->vmeslotnum,card->ipslotnum,
				card->regbase);
				isok=FALSE;
			}
			else
			{
				/* Card found Check PROM Contents */
				isok=(checkprom(card->regbase+PROM_OFFS,
								PROM_MODEL,
								1)!=0);
			}
		}

        /* If everything still OK - Check the Memory Address exsists */
		if (isok)
		{
			/* register the card in the A32 address space */
			card->membase=(int)ipmBaseAddr(	card->vmeslotnum,
											card->ipslotnum,
											ipac_addrMem);
      
			if (card->membase==NULL)
			{
				logMsg("%s: Cannot register hy8403 A32 device\n",
				(int)devstr,0,0, 0,0,0 );
				isok=FALSE;
			}
		}

        /* If everything still OK - Set up the ADC according to 'aisingle' */
		if (isok)
        {
			/* Set Up the Hy8403 to Read Thermocouples */
            if (card->aisingle == 3)
            { 
				/* Reset ADC1216 A */
	            while (!(*((word*)(card->regbase + REG_CSR)) & CSR_RDY));scowait();
                *(word*)(card->regbase + REG_ADCCON) = (CON_ADCA | CON_RESET);
	            
                /* Reset ADC1216 B */
	            while (!(*((word*)(card->regbase + REG_CSR)) & CSR_RDY));scowait();
                *(word*)(card->regbase + REG_ADCCON) = (CON_ADCB | CON_RESET);
	            scowait();
	            while (!(*((word*)(card->regbase + REG_CSR)) & CSR_RDY));scowait();
               
                /* Load ADC Mux Settings to ADC1216 A and ADC1216 B */
                wradc(card->regbase, CON_ADCA, ADC_MUX,		0x00);
                wradc(card->regbase, CON_ADCB, ADC_MUX,		0x00);
                /* Load ADC Setup Data to ADC1216 A and ADC1216 B */
                /* Buffer Enabled and Internal Ref set to 2.5V */
				wradc(card->regbase, CON_ADCA, ADC_SETUP, ADC_SETUP_BUFEN | ADC_SETUP_REFHI );
                wradc(card->regbase, CON_ADCB, ADC_SETUP, ADC_SETUP_BUFEN | ADC_SETUP_REFHI ); 
              
                /* Load Setup Data - Data Format UNIPOLAR, Sample Freq 50Hz, AUTO Filtering */
                /* i.e. send 0x0043 to ADC1216 A and ADC1216 B */
                wradc(card->regbase, CON_ADCA, ADC_DEC0, 0);
                wradc(card->regbase, CON_ADCB, ADC_DEC0, 0);
                wradc(card->regbase, CON_ADCA, ADC_DEC1, (ADC_DEC1_UB | ADC_SETTING_AUTO | 0x03));
                wradc(card->regbase, CON_ADCB, ADC_DEC1, (ADC_DEC1_UB | ADC_SETTING_AUTO | 0x03));
              
                /* Get ADC1216 A to Self Calibrate Offset and Gain */
                while (!(*((word*)(card->regbase + REG_CSR)) & CSR_RDY));scowait();
                *(word*)(card->regbase + REG_ADCCON) = 0x0004;
	            scowait();
	            while (!(*((word*)(card->regbase + REG_CSR)) & CSR_RDY));scowait();
                
                /* Get ADC1216 B to Self Calibrate Offset and Gain */
                while (!(*((word*)(card->regbase + REG_CSR)) & CSR_RDY));scowait();
                *(word*)(card->regbase + REG_ADCCON) = 0x8004;
	            scowait();
	            while (!(*((word*)(card->regbase + REG_CSR)) & CSR_RDY));scowait();
                
                /* Set Both Current Sources to Zero on Both ADC1216 A and ADC1216 B */
                /* Not Really Necessary but a precaution and ensures enough delay After Self Calibration */
                wradc(card->regbase, CON_ADCA, ADC_IDAC1, 0);
                wradc(card->regbase, CON_ADCB, ADC_IDAC1, 0);
                wradc(card->regbase, CON_ADCA, ADC_IDAC2, 0);
                wradc(card->regbase, CON_ADCB, ADC_IDAC2, 0); 

                /* Set Offset Voltage to -1.25V */
				/* ------- This must ONLY be done after the Self Calibration - otherwise value overwritten ------- */
                /* i.e. send 0x0043 to ADC1216 A and ADC1216 B */
                wradc(card->regbase, CON_ADCA, ADC_ODAC, ADC_ODAC_OFFSET_VOLTAGE);
	            scowait();
	            while (!(*((word*)(card->regbase + REG_CSR)) & CSR_RDY));scowait();
                wradc(card->regbase, CON_ADCB, ADC_ODAC, ADC_ODAC_OFFSET_VOLTAGE); 

                /* Setup PGA-MUX Registers */
                /* ----------------------- */

				/* Work through the number of channels on each ADC */
				for (locchan=0;locchan<card->numchantot;locchan++)
				{
					/*	Generate MUX Control + Gain...
						Bit 0-3  Negative Channel Select.
						Bit 4-7  Posative Channel Select.
						Bit 8-10 Mapped Gain Value 0-1 1-2 2-4 3-8 etc. */ 
					muxword= (PGAMuxSettings[locchan][card->aisingle])|(card->gainval<<8);
				
	                /* Load Channel with PGA Mux Settings */
		            while (!(*((word*)(card->regbase + REG_CSR)) & CSR_RDY));scowait();
			        *(word*)(card->regbase + REG_PGAMUX + idst) = muxword;
                
					/* Move Address to next Channel */
					idst+=2;
				}
               
                /* Set Number of Channels from each ADC to 8 and a clock rate of 2Hz */
                while (!(*((word*)(card->regbase + REG_CSR)) & CSR_RDY));scowait();
                *(word*)(card->regbase + REG_CLKRAT) = 0x7001;
               
                /* Set Range in Analogue Control Register */
                while (!(*((word*)(card->regbase + REG_CSR)) & CSR_RDY));scowait();
                *(word*)(card->regbase + REG_ACR) = 0x1818;
               
                /* Set the DAC to generate the output current */
                while (!(*((word*)(card->regbase + REG_CSR)) & CSR_RDY));scowait();
                *(word*)(card->regbase + REG_DACOUT) = 0xF0F0;
               
                /* Arm the ADCs */
	            /* while (!(*((word*)(card->regbase + REG_CSR)) & CSR_RDY));scowait();
                *(word*)(card->regbase + REG_CSR) = 0x8000; */
                
    			if (sem_init(&(card->csem),1,1)!=OK)
	    		{
		    		printf("ERROR SEM_INIT FAILED!\n");
			    	return ERR;
    			}
			    armadc(card,1);
               
            }   /* End of aisingle == 3 */
            
			/* Set Up the Hy8403 to Read PT-100 */
            else if (card->aisingle == 2)
            { 
				/* Reset ADC1216 A */
	            while (!(*((word*)(card->regbase + REG_CSR)) & CSR_RDY));scowait();
                *(word*)(card->regbase + REG_ADCCON) = (CON_ADCA | CON_RESET);
	            
                /* Reset ADC1216 B */
	            while (!(*((word*)(card->regbase + REG_CSR)) & CSR_RDY));scowait();
                *(word*)(card->regbase + REG_ADCCON) = (CON_ADCB | CON_RESET);
	            scowait();
	            while (!(*((word*)(card->regbase + REG_CSR)) & CSR_RDY));scowait();
               

                /* Load ADC Setup Data i.e. send 0x0600 to ADC1216 A and ADC1216 B */
                wradc(card->regbase, CON_ADCA, ADC_MUX,		0x00);
                wradc(card->regbase, CON_ADCB, ADC_MUX,		0x00);
                wradc(card->regbase, CON_ADCA, ADC_SETUP,	0x06);
                wradc(card->regbase, CON_ADCB, ADC_SETUP,	0x06);
              
              
                /* Load Setup Data - Issue 6 PCB Only */
                /* i.e. send 0x00FE to ADC1216 A and ADC1216 B */
                /* Set Bits 7-1 to Input and Bit 0 to Output */
                wradc(card->regbase, CON_ADCA, ADC_DIR,		0xFE);
                wradc(card->regbase, CON_ADCB, ADC_DIR,		0xFE);
                /* Drive Bit 0 to O */
                wradc(card->regbase, CON_ADCA, ADC_DIO,		0x00);
                wradc(card->regbase, CON_ADCB, ADC_DIO,		0x00);
              

                /* Load Setup Data - Data Format UNIPOLAR, Sample Freq 50Hz, AUTO Filtering */
                /* i.e. send 0x0043 to ADC1216 A and ADC1216 B */
                wradc(card->regbase, CON_ADCA, ADC_DEC0,	0x00);
                wradc(card->regbase, CON_ADCB, ADC_DEC0,	0x00);
                wradc(card->regbase, CON_ADCA, ADC_DEC1,	0x43);
                wradc(card->regbase, CON_ADCB, ADC_DEC1,	0x43);
              
	            
                /* Get ADC1216 A to Self Calibrate Offset and Gain */
	            while (!(*((word*)(card->regbase + REG_CSR)) & CSR_RDY));scowait();
                *(word*)(card->regbase + REG_ADCCON) = 0x0004;
	            scowait();
	            while (!(*((word*)(card->regbase + REG_CSR)) & CSR_RDY));scowait();
	            
                /* Get ADC1216 B to Self Calibrate Offset and Gain */
	            while (!(*((word*)(card->regbase + REG_CSR)) & CSR_RDY));scowait();
                *(word*)(card->regbase + REG_ADCCON) = 0x8004;
	            scowait();
	            while (!(*((word*)(card->regbase + REG_CSR)) & CSR_RDY));scowait();


                /* Setup PGA-MUX Registers */
                /* ----------------------- */

				/* Work through the number of channels on each ADC */
				for (locchan=0;locchan<card->numchantot;locchan++)
				{
					/*	Generate MUX Control + Gain...
						Bit 0-3  Negative Channel Select.
						Bit 4-7  Posative Channel Select.
						Bit 8-10 Mapped Gain Value 0-1 1-2 2-4 3-8 etc. */ 
					muxword= (PGAMuxSettings[locchan][card->aisingle])|(card->gainval<<8);
				
	                /* Load Channel with PGA Mux Settings */
		            while (!(*((word*)(card->regbase + REG_CSR)) & CSR_RDY));scowait();
			        *(word*)(card->regbase + REG_PGAMUX + idst) = muxword;
                
					/* Move Address to next Channel */
					idst+=2;
				}
               
                /* Set Number of Channels from each ADC to 8 and a clock rate of 2Hz */
	            while (!(*((word*)(card->regbase + REG_CSR)) & CSR_RDY));scowait();
                *(word*)(card->regbase + REG_CLKRAT) = 0x7001;
                
               
                /* Set Range in Analogue Control Register */
	            while (!(*((word*)(card->regbase + REG_CSR)) & CSR_RDY));scowait();
                *(word*)(card->regbase + REG_ACR) = 0x1818;
                
               
                /* Set the DAC to generate the output current */
	            while (!(*((word*)(card->regbase + REG_CSR)) & CSR_RDY));scowait();
                *(word*)(card->regbase + REG_DACOUT) = 0xF0F0;

               
                /* Arm the ADCs */
	            /* while (!(*((word*)(card->regbase + REG_CSR)) & CSR_RDY));scowait();
                *(word*)(card->regbase + REG_CSR) = 0x8000; */
                
    			if (sem_init(&(card->csem),1,1)!=OK)
	    		{
		    		printf("ERROR SEM_INIT FAILED!\n");
			    	return ERR;
    			}
			    armadc(card,1);
               
            }   /* End of aisingle == 2 */
            
            /* if aisingle = 0 or 1 */ 
            else
            {
                /* HW is OK, determine the CSR registers */
	    		csr=0;
		    	if (card->IPmemsize==1)csr |=CSR_1M;

			    #define __DOINT__ 1
    			#ifdef  __DOINT__
	    		/* ****initialise interrupts**** */
		    	scanIoInit(&card->ioscanpvt);
    			/* carrier card setup */
	    		status=ipmIntConnect(	card->vmeslotnum,
		    							card->ipslotnum,
			    						card->intvecnum,
				    					(VOIDFUNCPTR)hy8403_isr,
					    				card->cardnum);
    			if (status!=OK)
	    		{
		    		printf("%s: intConnect ERROR %d",devstr,
			    		card->vmeslotnum);
				    isok=FALSE;
			    }
      
    			ipmIrqCmd(	card->vmeslotnum,
	    					card->ipslotnum,
		    				0,
			    			ipac_irqEnable);

    			/* 
                    8403 specifics:
				    a) the vector number is written in dumpcard
				    b) program 8403 to generate interrupt on End of Scan
			    */
			    csr|=CSR_EE;
			    /* ****end of interrupt setup**** */
			    #endif

			    if (card->clocksource==1)  csr|=CSR_XC;
			    if (card->hwInhibEnable==1)csr|=CSR_ET;
      
			    if (card->trigger>0)
			    {
				    /* 
                        use a trigger:
					    a) number of conversions is trigger
					    b) program the CSR
				    */
				    card->sampreg=card->trigger;
				    csr|=CSR_EX;
			    }
			    else
			    {
				    card->sampreg=card->samples;
	    		}
			    card->csr=csr;

			    card->xilinxacr=(card->dacrange1<<3)|(card->dacrange2<<11);

    			/*
                    we have determined CSR, sampreg, clockrate, vecnum 
	    			and xilinxacr;
		    		a) reset and init the ADC's
			    	b) write values to the Xilinx
				    c) start xilinx scanning
			    */
    			if (sem_init(&(card->csem),1,1)!=OK)
	    		{
		    		printf("ERROR SEM_INIT FAILED!\n");
			    	return ERR;
    			}
		   
	    		initadc(card);
		    	if (dumpcard(card)!=OK)
			    {
				    printf("ERROR! dumpcard failed!\n");
				    return ERR;
			    }
			    armadc(card,1);

		    }  
 
			/*  
                this line only says that the given card
	    		was configured correctly
            */
			card->isconfigured=TRUE;

		}	/* if isok */
    
		if (isok)numcards++;

		card=card->next;

	}	/* End of while card!=NULL	*/

	printf("%s: FOUND A TOTAL OF %d CARDS\n\n",devstr,numcards);
	if (numcards==0)return ERR;

	return OK;
}



/**********************************************************

    NAME:  xilinxrdval()

    Description:

    Function - read a 24 bit ADC value from the Xilinx

    Parameters:

    Inputs:		card	 - Pointer to Structure for card
						   to read.	
				globchan - Which Analog Channel to read.
    Outputs:        
    Return:		The read 32 Bits ADC Value.

    Notes:

    globchan maps as follows.......

    globchan    ADC     Channel
        0       A          1
        1       A          2
        2       A          3
        3       A          4
        4       A          5
        5       A          6
        6       A          7
        7       A          8
        8       B          1
        9       B          2
        10      B          3
        11      B          4
        12      B          5
        13      B          6
        14      B          7
        15      B          8



    Programmer(s):  Darrell Nineham
                    Walter Scott            
                    Steve Hunt

**********************************************************

Revision History
----------------

Date            By      Version     Descripition
----            --      -------     ------------
19-OCT-2004     DAN     1.1.1.3     Converted to Hytec
                                    Coding Standard.
    
**********************************************************/
static unsigned long xilinxrdval(cardinfo* card, int globchan)
{
    volatile longword   reg         = card->regbase + REG_ADCBASE;      
    word			    numchanchip	= card->numchanchip;
	/* word			    unipolar	= card->unipolar; */
	word			    ndx;
	word			    lo,hi;
	unsigned long	    retval;


	/* begin */

	/*printf("XILINX RDVAL!\n");*/

	/* If Analog Input Debug Level above level 1 */ 
	if (debugai > 1)
	{
		/* Display Xilinx Registers */
		dumpxilinx(card);
	}

	/*	
        make sure we are armed, or the Xilinx ADC data
		registers will not be updated
		
        if (!isarmed(card))
        {
		    printf("xilinxrdval: XILINX HAS NOT BEEN ARMED  0x%04x!\n",
		            *((word*)(dstptr+REG_CSR)));
		    return ERR;
		}
	*/

    /* Convert Passed ADC Channel 0-15 to an actual Address */
	if (globchan < numchanchip)
	{
		ndx = globchan * 2;
	}
	else
	{
		ndx = (globchan-numchanchip) * 2 + 1;
	}
	reg += ndx * 4;

    /* Read bottom 16 bits */
	lo = *((word*)reg);
    
    /* printf("Address - 0x%8x - 0x%4x\n",reg, lo); */
    
    /* Move to High Byte */
    reg+=2;
    
    /* Read Top 16 bits */
	hi = *((word*)reg);
	
    /* Generate 32 Bit Value */
    retval = (hi<<16) + lo;

    /* Scaling Factor if set up for PT-100 */
    if (card->aisingle == 2)
    {
        retval = (retval + 1) * 2;
    }        

	/* mask off the top byte */
	retval &= 0x00ffffff;

    /* printf("Address - 0x%8x - 0x%4x\n",reg, hi); */

	/*	
        if we are in bipolar mode, we have to make sure that
		we always return an unsigned integer.
		Achieve this by toggling the leading bit
    */
	/* if (!unipolar)
	{
		retval ^= 0x800000;
	}*/

	/* make adjustments to the value due to gain */
	/* retval /= card->gain; */

	if (debugai > 0)
	{
	    /*
        printf("xilinxrdval %d (%d) returning 0x%08x %d\n",
				globchan,ndx,retval,retval);
        */
	}

	return retval;
}



/**********************************************************


	NAME:			ThermocoupleCalibrationTimerInterruptHandler()
	    
    Description:
        
    Function - Called every second, steps through different
	MUX settings and updates the zero, neg and pos full scale
	readings for both ADCs.  To keep the calibration values
	up to date.

    Parameters:
               
    Inputs:			arg1,arg2,arg3,arg4,arg5,
                    arg6,arg7,arg8,arg9,arg10 - Unused.

    Outputs:        bCalibrationCount updated 

    Return:         Nothing.
                            
    Notes:
                            
    Programmer(s):  Darrell Nineham

**********************************************************

 Revision History
 ----------------

 Date            By      Version     Descripition
 ----            --      -------     ------------
 21-NOV-2005     DAN     1.1.1.2     Intial Version.

***********************************************************/

static void ThermocoupleCalibrationTimerInterruptHandler(int arg1,
                                                         int arg2,
                                                         int arg3,
                                                         int arg4,
                                                         int arg5,
                                                         int arg6,
                                                         int arg7,
                                                         int arg8,
                                                         int arg9,
                                                         int arg10)
{
	cardinfo* card=cardlst;

    InterruptCount++;

	/* For each configured card... */
	while(card!=NULL)
	{
		/* Only if Set Up the Hy8403 to Read Thermocouples */
        if (card->aisingle == 3)
		{
			/* Setup PGA-MUX Registers to Calibrate Zero Volts Refernce */
            /* -------------------------------------------------------- */

			/*	
				Generate MUX Control + Gain...
				Bit 0-3  Negative Channel Select.
				Bit 4-7  Posative Channel Select.
				Bit 8-10 Mapped Gain Value 0-1 1-2 2-4 3-8 etc. 
			
				Assumes that Channel 8 on ADC A is its Calibration Voltage, and 
				Channel 7 on ADC B is its Calibration Voltage. Channel 8 on ADC B 
				is the PT100 Cold Junction reference.
			*/ 

			switch(bCalibrationCount)
			{	
				/* Read Zero Calibration then Set MUX for POS Full Scale */
				/* ----------------------------------------------------- */
				case 0:
					/* Read ADC A 'Zero Voltage' */
					card->adcAzero = xilinxrdval(card, 7);
					/* Read ADC B 'Zero Voltage' */
					card->adcBzero = xilinxrdval(card, 14);

					/* Setup PGA-MUX Registers to Calibrate Positive Reference */
		            /* ------------------------------------------------------- */
				    /* Load Last Channel but One on ADC A with PGA Mux Settings to Short Channel Input */
			        while (!(*((word*)(card->regbase + REG_CSR)) & CSR_RDY));scowait();
				    *(word*)(card->regbase + THERMOCOUPLE_CAL_A) = 0x0078 | (card->gainval<<8);

		            /* Load Last Channel on ADC B with PGA Mux Settings to Short Channel Input */
				    while (!(*((word*)(card->regbase + REG_CSR)) & CSR_RDY));scowait();
				    *(word*)(card->regbase + THERMOCOUPLE_CAL_B) = 0x0068 | (card->gainval<<8);

					/* Move to Next Step of Calibration */
					bCalibrationCount = 1;
					break;

				/* Read POS Calibration then Set MUX for NEG Full Scale */
				/* ---------------------------------------------------- */
				case 1:
		            /* Read ADC A 'Positive Voltage' */
					card->adcAposfs = xilinxrdval(card, 7);
					/* Read ADC B 'Positive Voltage' */
					card->adcBposfs = xilinxrdval(card, 14);

					/* Setup PGA-MUX Registers to Calibrate Negative Refernce */
		            /* ------------------------------------------------------ */
				    /* Load Last Channel but One on ADC A with PGA Mux Settings to Short Channel Input */
			        while (!(*((word*)(card->regbase + REG_CSR)) & CSR_RDY));scowait();
				    *(word*)(card->regbase + THERMOCOUPLE_CAL_A) = 0x0087 | (card->gainval<<8);

		            /* Load Last Channel on ADC B with PGA Mux Settings to Short Channel Input */
				    while (!(*((word*)(card->regbase + REG_CSR)) & CSR_RDY));scowait();
				    *(word*)(card->regbase + THERMOCOUPLE_CAL_B) = 0x0086 | (card->gainval<<8);
					
					/* Move to Next Step of Calibration */
					bCalibrationCount = 2;
					break;
				
				/* Read NEG Calibration then Set MUX for Zero Full Scale */
				/* ----------------------------------------------------- */
				case 2: 
		            /* Read ADC A 'Negative Voltage' */
					card->adcAnegfs = xilinxrdval(card, 7);
					/* Read ADC B 'Negative Voltage' */
					card->adcBnegfs = xilinxrdval(card, 14);

					/* Setup PGA-MUX Registers to Calibrate Zero Volts Reference */
				    /* --------------------------------------------------------- */
					/* Load Last Channel but One on ADC A with PGA Mux Settings to Short Channel Input */
					while (!(*((word*)(card->regbase + REG_CSR)) & CSR_RDY));scowait();
					*(word*)(card->regbase + THERMOCOUPLE_CAL_A) = 0x0088 | (card->gainval<<8);

					/* Load Last Channel on ADC B with PGA Mux Settings to Short Channel Input */
					while (!(*((word*)(card->regbase + REG_CSR)) & CSR_RDY));scowait();
					*(word*)(card->regbase + THERMOCOUPLE_CAL_B) = 0x0088 | (card->gainval<<8);

					/* Move to Next Step of Calibration */
					bCalibrationCount = 0;
					break;

				/* Read NEG Calibration then Set MUX for Zero Full Scale */
				/* ----------------------------------------------------- */
				default:
					/* Setup PGA-MUX Registers to Calibrate Zero Volts Reference */
				    /* --------------------------------------------------------- */
					/* Load Last Channel but One on ADC A with PGA Mux Settings to Short Channel Input */
					while (!(*((word*)(card->regbase + REG_CSR)) & CSR_RDY));scowait();
					*(word*)(card->regbase + THERMOCOUPLE_CAL_A) = 0x0088 | (card->gainval<<8);

					/* Load Last Channel on ADC B with PGA Mux Settings to Short Channel Input */
					while (!(*((word*)(card->regbase + REG_CSR)) & CSR_RDY));scowait();
					*(word*)(card->regbase + THERMOCOUPLE_CAL_B) = 0x0088 | (card->gainval<<8);

					/* Move to Next Step of Calibration */
					bCalibrationCount = 0;
					break;
			}

			/* Work Out Calbration */
            /*
			card->adcApsFS = (longword)((float)(card->adcAposfs - card->adcAzero) * (float)(0x800000)/(float)(CALBITS)) + card->adcAzero;
			card->adcBpsFS = (longword)((float)(card->adcBposfs - card->adcBzero) * (float)(0x800000)/(float)(CALBITS)) + card->adcBzero;
			card->adcAngFS = card->adcAzero - (longword)((float)(card->adcAzero - card->adcAnegfs) * (float)(0x800000)/(float)(CALBITS));
			card->adcBngFS = card->adcBzero - (longword)((float)(card->adcBzero - card->adcBnegfs) * (float)(0x800000)/(float)(CALBITS));
            */

		} /* End of aisingle = 3 (Thermocouple) Check */ 
	
		/* Move to the Next Card */
		card=card->next;

	} /* End of while card!=NULL */

}    

             

/**********************************************************

    NAME:  CALIBRATE()

    Description:

    Function - 
	Loop through the cardlst and calibrate all devices in
	the list setup for Thermocouples.

    Parameters:

    Inputs:        None.
    Outputs:        
    Return:        None. 

    Notes:

    Programmer(s):  Darrell Nineham

**********************************************************

Revision History
----------------

Date            By      Version     Descripition
----            --      -------     ------------
21-NOV-2005     DAN     1.1.1.2		Added Task to check
									Thermocouple Calibration
									Values.
   
19-AUG-2005     DAN     1.1.1.1		Initial Version.
    

**********************************************************/
void CALIBRATE()
{
	cardinfo* card=cardlst;
	unsigned char bThermoCoupleDetect = FALSE;
    int argument = 0;

	/* For each configured card... */
	while(card!=NULL)
	{
		/* Only if Set Up the Hy8403 to Read Thermocouples */
        if (card->aisingle == 3)
		{
			/* Card Configured for Thermocouples Detected */
			bThermoCoupleDetect = TRUE;

			/* Setup PGA-MUX Registers to Calibrate Zero Volts Reference */
            /* --------------------------------------------------------- */

			/*	
				Generate MUX Control + Gain...
				Bit 0-3  Negative Channel Select.
				Bit 4-7  Posative Channel Select.
				Bit 8-10 Mapped Gain Value 0-1 1-2 2-4 3-8 etc. 
			
				Assumes that Channel 8 on ADC A is its Calibration Voltage, and 
				Channel 7 on ADC B is its Calibration Voltage. Channel 8 on ADC B 
				is the PT100 Cold Junction reference.
			*/ 


            /* Load Last Channel but One on ADC A with PGA Mux Settings to Short Channel Input */
	        while (!(*((word*)(card->regbase + REG_CSR)) & CSR_RDY));scowait();
		    *(word*)(card->regbase + THERMOCOUPLE_CAL_A) = 0x0088 | (card->gainval<<8);

            /* Load Last Channel on ADC B with PGA Mux Settings to Short Channel Input */
	        while (!(*((word*)(card->regbase + REG_CSR)) & CSR_RDY));scowait();
		    *(word*)(card->regbase + THERMOCOUPLE_CAL_B) = 0x0088 | (card->gainval<<8);
            
            taskDelay(CLOCKS_PER_SEC); 
				
			/* Read ADC A 'Zero Voltage' */
			card->adcAzero = xilinxrdval(card, 7);
			/* Read ADC B 'Zero Voltage' */
			card->adcBzero = xilinxrdval(card, 14);
		

			/* Setup PGA-MUX Registers to Calibrate Positive Reference */
            /* ------------------------------------------------------- */
            /* Load Last Channel but One on ADC A with PGA Mux Settings to Short Channel Input */
	        while (!(*((word*)(card->regbase + REG_CSR)) & CSR_RDY));scowait();
		    *(word*)(card->regbase + THERMOCOUPLE_CAL_A) = 0x0078 | (card->gainval<<8);

            /* Load Last Channel on ADC B with PGA Mux Settings to Short Channel Input */
	        while (!(*((word*)(card->regbase + REG_CSR)) & CSR_RDY));scowait();
		    *(word*)(card->regbase + THERMOCOUPLE_CAL_B) = 0x0068 | (card->gainval<<8);
				
            taskDelay(CLOCKS_PER_SEC); 
			
            /* Read ADC A 'Positive Voltage' */
			card->adcAposfs = xilinxrdval(card, 7);
			/* Work Out Calbration */
			card->adcApsFS = (longword)((float)(card->adcAposfs - card->adcAzero) * (float)(0x800000)/(float)(CALBITS)) + card->adcAzero;

			/* Read ADC B 'Positive Voltage' */
			card->adcBposfs = xilinxrdval(card, 14);
			/* Work Out Calbration */
			card->adcBpsFS = (longword)((float)(card->adcBposfs - card->adcBzero) * (float)(0x800000)/(float)(CALBITS)) + card->adcBzero;
			
            
			/* Setup PGA-MUX Registers to Calibrate Negative Reference */
            /* ------------------------------------------------------- */
            /* Load Last Channel but One on ADC A with PGA Mux Settings to Short Channel Input */
	        while (!(*((word*)(card->regbase + REG_CSR)) & CSR_RDY));scowait();
		    *(word*)(card->regbase + THERMOCOUPLE_CAL_A) = 0x0087 | (card->gainval<<8);

            /* Load Last Channel on ADC B with PGA Mux Settings to Short Channel Input */
	        while (!(*((word*)(card->regbase + REG_CSR)) & CSR_RDY));scowait();
		    *(word*)(card->regbase + THERMOCOUPLE_CAL_B) = 0x0086 | (card->gainval<<8);

            taskDelay(CLOCKS_PER_SEC);
			
            /* Read ADC A 'Negative Voltage' */
			card->adcAnegfs = xilinxrdval(card, 7);
			/* Work Out Calbration  */
			card->adcAngFS = card->adcAzero - (longword)((float)(card->adcAzero - card->adcAnegfs) * (float)(0x800000)/(float)(CALBITS));

			/* Read ADC B 'Negative Voltage' */
			card->adcBnegfs = xilinxrdval(card, 14);
			/* Work Out Calbration  */
			card->adcBngFS = card->adcBzero - (longword)((float)(card->adcBzero - card->adcBnegfs) * (float)(0x800000)/(float)(CALBITS));


			/* Display Calibration Values - If a bit of Debug required */
			/* 
			printf("Card %d - ADC A Zero - 0x%6X\n", card->cardnum, card->adcAzero);
			printf("Card %d - ADC B Zero - 0x%6X\n", card->cardnum, card->adcBzero);
			printf("Card %d - ADC A Pos  - 0x%6X\n", card->cardnum, card->adcAposfs);
			printf("Card %d - ADC B Pos  - 0x%6X\n", card->cardnum, card->adcBposfs);
			printf("Card %d - ADC A Neg  - 0x%6X\n", card->cardnum, card->adcAnegfs);
			printf("Card %d - ADC B Neg  - 0x%6X\n", card->cardnum, card->adcBnegfs);

			printf("Card %d - ADC A Pos FS - 0x%6X\n", card->cardnum, card->adcApsFS);
			printf("Card %d - ADC B Pos FS - 0x%6X\n", card->cardnum, card->adcBpsFS);
			printf("Card %d - ADC A Neg FS - 0x%6X\n", card->cardnum, card->adcAngFS);
			printf("Card %d - ADC B Neg FS - 0x%6X\n", card->cardnum, card->adcBngFS); */
            
		} /* End of aisingle = 3 (Thermocouple) Check */ 
	
		/* Move to the Next Card */
		card=card->next;

	} /* End of while card!=NULL */


	/* If Any Cards Configured for Thermocouples Detected */
	if (bThermoCoupleDetect == TRUE)
	{
        /* Set Up Timer - To Update Calibration once a second */
        taskSpawn(  "drvHy8403:ThermocoupleCalTimer",
					3,
                    VX_FP_TASK,
                    1000,
                    (FUNCPTR)&ThermocoupleCalibrationTimerInterruptHandler,
                    argument,
                    argument,
                    argument,
                    argument,
                    argument,
                    argument,
                    argument,
                    argument,
                    argument,
                    argument);
	} /* End of If Thermocouple Detected */
}



/**********************************************************

    NAME:  calval()

    Description:

    Function - 
	Converts a 'raw' ADC Value read from a thermocouple to
	a calibrated value. To allow access to calibration values
    channels past the end of the ADC channels are used.

    Parameters:

    Inputs:        cardinfo* card - Pointer to Card Data 
                                    Structure.
                   int globchan   - Which ADC Channel or
                                    Calibration Value.
                   longword raw   - Uncalibrated ADC Value.
    Outputs:       None. 
    Return:        Required Calibrated Value or the Actual
                   Calibration Value.

    Notes:

    Programmer(s):  Darrell Nineham

**********************************************************

Revision History
----------------

Date            By      Version     Descripition
----            --      -------     ------------
21-NOV-2005     DAN     1.1.1.2     Ability to return
                                    calibration values 
                                    added.

09-SEP-2005     DAN     1.8			Initial Version.
    

**********************************************************/
static longword calval(cardinfo* card, int globchan, longword raw)
{
	/* If 'raw' data from ADC A */ 
	if (globchan < card->numchanchip)	
	{
		if (raw > card->adcAzero)
		{
			return( (longword) (((float)(raw - card->adcAzero)*(float)(0x800000))/(float)(card->adcApsFS - card->adcAzero)) + 0x800000 );
		}
		else
		{
			return( 0x800000 - (longword) (((float)(card->adcAzero - raw)*(float)(0x800000))/(float)(card->adcAzero - card->adcAngFS)));
		}
	}
	/* else If 'raw' data from ADC B */
	else if (globchan <  card->numchantot)
	{
		if (raw > card->adcBzero)
		{
			return( (longword) (((float)(raw - card->adcBzero)*(float)(0x800000))/(float)(card->adcBpsFS - card->adcBzero)) + 0x800000 );
		}
		else
		{
			return( 0x800000 - (longword) (((float)(card->adcBzero - raw)*(float)(0x800000))/(float)(card->adcBzero - card->adcBngFS)));
		}
	}
	/* else Return Thermocouples Calibration Values */
	else 
	{
        switch(globchan)
        { 
            case ADC_A_ZERO_CAL: 
	            return ( card->adcAzero );	    /* ADC A - Zero Raw Calibration */
                break;
                
            case ADC_A_POS_CAL: 
	            return ( card->adcAposfs );	    /* ADC A - Pos  Raw Calibration */
                break;
	    
            case ADC_A_NEG_CAL: 
                return ( card->adcAnegfs );	    /* ADC A - Neg  Raw Calibration */
                break;
	    
            case ADC_B_ZERO_CAL: 
                return ( card->adcBzero );	    /* ADC B - Zero Raw Calibration */
                break;
	    
            case ADC_B_POS_CAL: 
                return ( card->adcBposfs );	    /* ADC B - Pos  Raw Calibration */
                break;
	    
            case ADC_B_NEG_CAL: 
                return ( card->adcBnegfs );     /* ADC B - Neg  Raw Calibration */
                break;
                
            default: 
                return ( 0 );                   /* Unknown Request - Return 0 */
                break;
        } /* End of Switch on Channel Number */
	}
}


/**********************************************************

    NAME:  adcrdval()

    Description:

    Function - read a 24 bit ADC value directly from the ADC.
			We have to:
		b) switch the MUX on the ADC to the correct channel
		d) read the data.

    Parameters:

    Inputs:         None.
    Outputs:        
    Return:         

    Notes:

    Programmer(s):  Darrell Nineham
                    Walter Scott            
                    Steve Hunt

**********************************************************

Revision History
----------------

Date            By      Version     Descripition
----            --      -------     ------------
19-OCT-2004     DAN     1.1.1.3     Converted to Hytec
                                    Coding Standard.
    
**********************************************************/

#define NUMTICKS 16

static unsigned long adcrdval(cardinfo* card, int globchan)
{
	word adcsel,i,ii;
	word unipolar=card->unipolar;
	volatile longword reg=card->regbase;
	unsigned long val;
	byte muxbyte;
	static word docall=20;
	unsigned int start;
	int locchan;
  
	/* begin */
	if (globchan< card->numchanchip)
	{
		adcsel=CON_ADCA;
		locchan=globchan;
	}
	else
	{
		adcsel=CON_ADCB;
		locchan=globchan-card->numchanchip;
	}
	/* select the MUX on the ADC to input this channel */
	muxbyte=(byte)(locchan<<5)+(locchan*2+1);
	wradc(reg,adcsel,ADC_MUX,muxbyte);
	/* printf("globchan %d, locchan %d, muxbyte 0x%2x\n",
		globchan,locchan,muxbyte);
	*/
  
	if (debugai > 0)
	{
		for (ii=0;ii<2;ii++)
		{
			for (i=0;i<NUMADCREG;i++)
			{
				printf("ADC REG %2d: 0x%2x\n",i,rdadc(reg,adctab[ii],i));
			}
		}
	}

	/* wait for the ADC to settle */
	start=tickGet();
	while (tickGet()-start <NUMTICKS);

	 
	/* now read the data from the ADC registers */
	*((word*)(reg+REG_ADCCON))=adcsel | CON_RDAT;
	while ( (*((word*)(reg+REG_CSR)) & CSR_RDY) ==0);scowait();

	val=((*((word*)(reg+REG_ADCCON))&0x00FF)<<16)+
				*((word*)(reg+REG_ADCDAT));
	/*printf("read  0x%08lx\n",val);*/
  
	/*	if we are in bipolar mode, we have to make sure that
		we always return an unsigned integer.
		Achieve this by toggling the leading bit */
	if (!unipolar)
	{
		val ^= 0x800000;
	}

	docall--;
	if (docall==0)
	{
		/* now perform a calibration
		while (!(*((word*)(reg+REG_CSR)) & CSR_RDY));scowait();
		*((word*)(reg+REG_ADCCON))=adcsel | CON_CALOG;
		while (!(*((word*)(reg+REG_CSR)) & CSR_RDY));scowait();
		*/
		docall=20;
	}


	/* make adjustments to the value due to gain */
	val /=card->gain;


	return val;
}


/**********************************************************

    NAME:  READ_AI()

    Description:

    Function - Analog Input


    Parameters:

    Inputs:		cardnum - Which Card.
				channel - Which Analog Input Channel.
				*parm	- String of Config Parameters.
				*val	- Pointer to location for result.

    Outputs:        
    Return:         

    Notes:

    Programmer(s):  Darrell Nineham
                    Walter Scott            
                    Steve Hunt

**********************************************************

Revision History
----------------

Date            By      Version     Descripition
----            --      -------     ------------
21-NOV-2005     DAN     1.1.1.2     Ability to return
                                    calibration values 
                                    added.

19-OCT-2004     DAN     1.1.1.1     Converted to Hytec
                                    Coding Standard.
    
**********************************************************/

#define MAXNUM 11

int READ_AI(word cardnum,
			word channel,
			char *parm,
			unsigned long *val)
{
	cardinfo*	card=cardlst;
	longword	base;
	int 		i,j,value,spacing;
	int 		n=MAXNUM-1;			/* number of maxavg */
	longword	samples,samples1,samples2,sampspace;
	longword	size,adr;
	longword	max=0;
	longword	min=MAXLONGWORD;
	longword	chanbase;
	longword	avg,err;
	longword	sig2=0;
	longword	highest[MAXNUM];
	word		offset,hi,lo;
	double		tot=0.0;

	/* begin */

	/* If Analog Input Debug enabled */
	if (debugai > 0)
	{
		printf("READ_AI: cardnum %d, channel %d parm %s\n",
				cardnum,channel,parm);
	}
  
	/* If the Passed Card Number Not Valid Exit */
	while(card!=NULL && card->cardnum!=cardnum)card=card->next;
	if (card==NULL)
	{
		printf("READ_AI: cardnum %d not found!\n",cardnum);
		return S_drv_noDevice;
	}
  
	/* If Card is NOT configured Exit */
	if (card->isconfigured==0)
	{
		printf("READ_AI: cardnum %d not configured\n",cardnum);
		return S_drv_noDevice;
	}
  
	/* If Card is NOT present Exit */
	if (ispresent(card) == 0)
	{
		printf("READ_AI: card %d not present\n",cardnum);
		return S_drv_noDevice;
	}
  
	size	  = card->memperchan;
	samples	  = card->samples;
	spacing	  = card->spacing;
	sampspace = samples*spacing;
  
	/* If a single sample */
	if (samples <= 1)
	{
		/* Read ADC Value Directly from Xilinx Registers */
		if (channel < card->numchantot)
        {
            *val = xilinxrdval(card,channel);
        }
        
		/* If Analog Input Debug is enabled */
		if (debugai > 0)
		{
			/* Display Value Read from Xilinx Register */
			printf("READ_AI: %ld\n",*val);
		}
		
		/* If an Error is returned exit */
		if (*val==ERR)
		{	
			return ERR;
		}

		/* If reading Thermocouples calibrate values first */
        /* Also Used to Access Thermocouple Calibration Values */
		if (card->aisingle == 3)
		{	
			*val = calval(card, channel, *val);
		}
	}
	else
	{
		/* THIS IS NOT YET IMPLEMENTED */
		printf("ERRORRORRORR!! MEMORY READ NOT IMPLEMENTED!\n");
		return S_drv_badParam;

		/* read from memory */
		offset=*((word*)(card->regbase+REG_CONVP));
		chanbase=card->membase+(channel*size);
		if (sampspace > offset)
		{
			samples1=offset / spacing ;
			samples2=samples-samples1;
			if(strstr(parm,"MAXAVG"))
			{
				for(j=0;j<=n;j++)highest[j]=0;
				for(i=0;i<samples1;i++)
				{
					adr=chanbase+(offset-(i*spacing))*2;
					hi=*((word*)(adr));lo=*((word*)(adr+1));
					value=lo+(hi<<BITSPERWORD);
	  
					for(j=n-1;j>=0 ;j--)
					{
						if(value > highest[j])
						{
							highest[j+1]=highest[j];
							highest[j]=value;
						}
						else
						{
							break;
						}
					}
				}

				for(i=0;i<samples2;i++)
				{
					adr=chanbase+(size-1-(i*spacing))*2;
					hi=*((word*)(adr));lo=*((word*)(adr+1));
					value=lo+(hi<<BITSPERWORD);
					for(j=n-1;j>=0 ;j--)
					{
						if(value > highest[j])
						{
							highest[j+1]=highest[j];
							highest[j]=value;
						}
						else
						{
							break;
						}
					}
				}
				
				for(j=0;j<n;j++)max+=highest[j];
				*val=max/n;
			}
			else if(strstr(parm,"MAX"))
			{
				for(i=0;i<samples1;i++)
				{
					adr=chanbase+(offset-(i*spacing))*2;
					hi=*((word*)(adr));lo=*((word*)(adr+1));
					value=lo+(hi<<BITSPERWORD);
					if(max < value)max=value;
				}
	
				for(i=0;i<samples2;i++)
				{
					adr=chanbase+(size-1-(i*spacing))*2;
					hi=*((word*)(adr));lo=*((word*)(adr+1));
					value=lo+(hi<<BITSPERWORD);
					if(max < value)max=value;
				}
				*val=max;
			}
			else if(strstr(parm,"MIN"))
			{
				for(i=0;i<samples1;i++)
				{
					adr=chanbase+(offset-(i*spacing))*2;
					hi=*((word*)(adr));lo=*((word*)(adr+1));
					value=lo+(hi<<BITSPERWORD);
					if(min > value)min=value;
				}
				for(i=0;i<samples2;i++)
				{
					adr=chanbase+(size-1-(i*spacing))*2;
					hi=*((word*)(adr));lo=*((word*)(adr+1));
					value=lo+(hi<<BITSPERWORD);
					if(min > value)min=value;
				}
			}
			else if(strstr(parm,"RMS"))
			{
				for(i=0;i<samples1;i++)
				{
					adr=chanbase+(offset-(i*spacing))*2;
					hi=*((word*)(adr));lo=*((word*)(adr+1));
					tot+=lo+(hi<<BITSPERWORD);
				}
				for(i=0;i<samples2;i++)
				{
					adr=chanbase+(size-1-(i*spacing))*2;
					hi=*((word*)(adr));lo=*((word*)(adr+1));
					tot+=lo+(hi<<BITSPERWORD);
				}
				avg=tot/samples;
				for(i=0;i<samples1;i++)
				{
					adr=chanbase+(offset-(i*spacing))*2;
					hi=*((word*)(adr));lo=*((word*)(adr+1));
					err=avg-(lo+(hi<<BITSPERWORD));
					sig2+= err*err;
				}
				for(i=0;i<samples2;i++)
				{
					adr=chanbase+(size-1-(i*spacing))*2;
					hi=*((word*)(adr));lo=*((word*)(adr+1));
					err=avg-(lo+(hi<<BITSPERWORD));
					sig2+= err*err;
				}
				if(samples > 30)
				{
					*val=sqrt(sig2/samples);
				}
				else
				{
					*val=sqrt(sig2/(samples-1));
				}
			}
			else
			{
				for(i=0;i<samples1;i++)
				{
					adr=chanbase+(offset-(i*spacing))*2;
					hi=*((word*)(adr));lo=*((word*)(adr+1));
					tot+=(lo+(hi<<BITSPERWORD));
				}
				for(i=0;i<samples2;i++)
				{
					adr=chanbase+(size-1-(i*spacing))*2;
					hi=*((word*)(adr));lo=*((word*)(adr+1));
					tot+=(lo+(hi<<BITSPERWORD));
				}
				*val=tot / samples;
			}
		}
		else
		{
			/*samples*spacing < offset*/
			if(strstr(parm,"MAXAVG"))
			{
				for(j=0;j<=n;j++)highest[j]=0;
				for(i=0;i<samples;i++)
				{
					adr=chanbase+(offset-(i*spacing))*2;
					hi=*((word*)(adr));lo=*((word*)(adr+1));
					value=lo+(hi<<BITSPERWORD);
					for(j=n-1;j>=0 ;j--)
					{
						if(value > highest[j])
						{
							highest[j+1]=highest[j];
							highest[j]=value;
						}
						else
						{
							break;
						}
					}
				}
				for(j=0;j<n;j++)max+=highest[j];
				*val=max / n;
			}
			else if(strstr(parm,"MAX"))
			{
				for(i=0;i<samples;i++)
				{
					adr=chanbase+(offset-(i*spacing))*2;
					hi=*((word*)(adr));lo=*((word*)(adr+1));
					value=lo+(hi<<BITSPERWORD);
					if(max < value)max=value;
				}
				*val=max; 
			}
			else if(strstr(parm,"MIN"))
			{
				for(i=0;i<samples;i++)
				{
					adr=chanbase+(offset-(i*spacing))*2;
					hi=*((word*)(adr));lo=*((word*)(adr+1));
					value=lo+(hi<<BITSPERWORD);
					if(min > value)min=value;
				}
				*val=min; 
			}
			else if(strstr(parm,"RMS"))
			{
				for(i=0;i<samples;i++)
				{
					adr=chanbase+(offset-(i*spacing))*2;
					hi=*((word*)(adr));lo=*((word*)(adr+1));
					tot+=lo+(hi<<BITSPERWORD);
				}
				avg=tot/samples;
				sig2=0;
				for(i=0;i<samples;i++)
				{
					adr=chanbase+(offset-(i*spacing))*2;
					hi=*((word*)(adr));lo=*((word*)(adr+1));
					err=avg-(lo+(hi<<BITSPERWORD));
					sig2+= err*err;
				}
				if(samples > 30)
				{	
					*val=sqrt(sig2/samples);
				}
				else            
				{	
					*val=sqrt(sig2/(samples-1));
				}
			}
			else
			{
				for(i=0;i<samples;i++)
				{
					tot+=*(word*)(base+offset-(i*spacing));
				}
				*val=tot/samples;
			}
		}
	}
	return S_drv_OK;
}


/**********************************************************

    NAME:  adcwrite_ao()

    Description:

    Function - Analogue Output

    Parameters:

    Inputs:         None.
    Outputs:        
    Return:         

    Notes:

    Programmer(s):  Darrell Nineham
                    Walter Scott            
                    Steve Hunt

**********************************************************

Revision History
----------------

Date            By      Version     Descripition
----            --      -------     ------------
19-OCT-2004     DAN     1.1.1.3     Converted to Hytec
                                    Coding Standard.
    
**********************************************************/
static int adcwrite_ao(	unsigned short cardnum,
						unsigned short channel,
						unsigned int val)
{
	cardinfo*			card=cardlst;
	volatile longword	reg;
	word				adcsel;
	int					restart;
	byte				bval;
  
	/* begin */

	/*	verify that the card and signal are valid
		and do the hotswap check */
	if (debugao >0)
	{
		printf("WRITE_AO: cardnum %d, channel %d, val %d (0x%3x)\n",
								cardnum,channel,val,val);
	}
	
	while(card!=NULL && card->cardnum!=cardnum)card=card->next;
	if (card==NULL)
	{
		printf("WRITE_AO: cardnum %d not found!\n",cardnum);
		return S_drv_noDevice;
	}
  
	if (card->isconfigured==0)
	{
		printf("WRITE_AO: cardnum %d not configured\n",cardnum);
		return S_drv_noDevice;
	}
  
	if (ispresent(card)==0)
	{
		if (debugao>0)
		{
			printf("WRITE_AO: card %d not present\n",cardnum);
		}
		return S_drv_noDevice;
	}
  
	if (channel >=NUMAOCHAN)
	{
		printf("WRITE_AO: illegal channel number %d\n",channel);
		return S_drv_noDevice;
	}
  
	if (val > 255)
	{
		printf("WRITE_AO: value written is too big (max 255) %d\n",val);
		return S_drv_badParam;
	}
  
	bval=val;
	reg=card->regbase;

	/*	now do something. The first DAC on each chip is used, so that
		channel must select chip A or B.
		Always write to IDAC1 on each chip.
		IDAC2 of neither chip is used */

	/*	if the Xilinx is reading in ADC values,
		stop it and restart it after we have set the DAC
	*/
	restart=isarmed(card);
	if (restart)
	{
		if (debugao)printf("stopping ADC scanning\n");
		armadc(card,0);
	}

	if (debugao)
	{
		dumpxilinx(card);
		dumpadc(card);
	}

	/* set the DAC value */
	if (channel==0)adcsel=CON_ADCA;
	else adcsel=CON_ADCB;
  
	wradc(reg,adcsel,ADC_IDAC1,bval);
  
	if (debugao)
	{
		printf("WRITE_AO: OK!\n");
		dumpadc(card);
		scowait();
	}

	/* restart scanning if necessary */
	if (restart)
	{
		if (debugao)printf("restarting scanning\n");
		armadc(card,1);
	}

	return S_drv_OK;
}


/**********************************************************

    NAME:  WRITE_AO()

    Description:

    Function -	this is the version that writes directly to
				the Xilinx register


    Parameters:

    Inputs:         None.
    Outputs:        
    Return:         

    Notes:

    Programmer(s):  Darrell Nineham
                    Walter Scott            
                    Steve Hunt

**********************************************************

Revision History
----------------

Date            By      Version     Descripition
----            --      -------     ------------
19-OCT-2004     DAN     1.1.1.3     Converted to Hytec
                                    Coding Standard.
    
**********************************************************/
int WRITE_AO(	unsigned short cardnum,
				unsigned short channel,
				unsigned int val)
{
	cardinfo*		card=cardlst;
	volatile word*	reg;
	word			wval;

	/* begin */

	/*	verify that the card and signal are valid
		and do the hotswap check*/
	if (debugao >0)
	{
		printf("WRITE_AO: cardnum %d, channel %d, val %d (0x%3x)\n",
									cardnum,channel,val,val);
	}
  
	while(card!=NULL && card->cardnum!=cardnum)card=card->next;
	if (card==NULL)
	{
		printf("WRITE_AO: cardnum %d not found!\n",cardnum);
		return S_drv_noDevice;
	}
  
	if (card->isconfigured==0)
	{
		printf("WRITE_AO: cardnum %d not configured\n",cardnum);
		return S_drv_noDevice;
	}
  
	if (ispresent(card)==0)
	{
		if (debugao>0)
		{
			printf("WRITE_AO: card %d not present\n",cardnum);
		}
		return S_drv_noDevice;
	}
  
	if (channel >=NUMAOCHAN)
	{
		printf("WRITE_AO: illegal channel number %d\n",channel);
		return S_drv_noDevice;
	}
  
	if (val >255)
	{
		printf("WRITE_AO: value written is too big (max 255) %d\n",val);
		return S_drv_badParam;
	}
  
	wval=val;
	reg=(word*)(card->regbase+REG_DACOUT);


	/*	now do something.
		Write to the appropriate Xilinx register */
	/*	set the DAC value, making sure to preserve the 'other byte'
    channel 0 is the lower byte,
    channel 1 is the higher byte
	*/

	if (debugao)
	{
		printf("DACOUT REG BEFORE 0x%04x\n",*reg);
	}
  
	if (channel==0)
	{
		/* set the lower byte, preserve the higher byte */
		*reg=(*reg & 0xFF00)| wval;
	}
	else
	{
		/* preserve the lower byte, set the higher byte */
		*reg=(*reg & 0x00FF)|(wval <<8);
	}
 
	if (debugao)
	{
		printf("DACOUT REG AFTER 0x%04x\n",*reg);
	}
 
	if (debugao)
	{
		printf("WRITE_AO: OK!\n");
	}
	return S_drv_OK;
}



/**********************************************************

    NAME:  READ_WF()

    Description:

    Function -	Waveform Input


    Parameters:

    Inputs:         None.
    Outputs:        
    Return:         

    Notes:

    Programmer(s):  Darrell Nineham
                    Walter Scott            
                    Steve Hunt

**********************************************************

Revision History
----------------

Date            By      Version     Descripition
----            --      -------     ------------
19-OCT-2004     DAN     1.1.1.3     Converted to Hytec
                                    Coding Standard.
    
**********************************************************/
int READ_WF(word cardnum, 
			word channel,
			char *parm,
			unsigned long *val,
			unsigned long nelm)
{
	cardinfo*	card=cardlst;
	word*		base;
	word 		offset;
	int 		samples;
	longword	sum;
	longword*	data;
	int			i,j;
  
	/* begin */

	/*	verify that the card and signal are valid,
		and do the hotswap check */
	if (debugai >0)
	{
		printf("READ_WF: cardnum %d, channel %d parm %s\n",
						cardnum,channel,parm);
	}

	while(card!=NULL && card->cardnum!=cardnum)card=card->next;
  
	if (card==NULL)
	{
		printf("READ_WF: cardnum %d not found!\n",cardnum);
		return S_drv_noDevice;
	}
  
	if (card->isconfigured==0)
	{
		printf("READ_WF: cardnum %d not configured\n",cardnum);
		return S_drv_noDevice;
	}
  
	if (ispresent(card)==0)
	{
		if (debugai>0)
		{
			printf("READ_WF: card %d not present\n",cardnum);
		}
		return S_drv_noDevice;
	}
  
	/* ********** CHECK channel HERE ********** */


	base=(word*)(card->membase+(card->memperchan*channel));
	offset=*((word*)(card->regbase+REG_CONVP));

	if(strstr(parm,"SMOO"))
	{
		samples=card->samples;
		data=(longword*)(base+offset-(samples*nelm)*2);
		for(i=0;i<nelm;i++)
		{
			sum=0;
			for(j=0;j<samples;j++)sum+=data[i*samples+j];
			val[i]=(longword)sum/samples;
		}
	}
	else
	{
		if (nelm > offset)
		{
			memmove(val,base+(offset-nelm)*2,
				(nelm-offset)*sizeof(longword));
			memmove(val,base,offset*sizeof(longword));
		}
		else
		{
			memmove(val,base+(offset-nelm)*2,
					nelm*sizeof(longword));
		}
	}
	return S_drv_OK;
}


/**********************************************************

    NAME:  CONFIGURE()

    Description:

    Function -

    Parameters:

    Inputs:
                
    int cardnum     -   card number as used in INP fields
	int vmeslotnum  -   vmeslot of the carrier board
	int ipslotnum   -   IP slot number
	int vectornum   -   interrupt vector number. 0=disabled
	int aitype      -   0=differential,
                        1=single ended,
                        2=PT100,
                        3=Thermocouple 
	int IPmemsize   -   1=1MB memory, 2=2MB memory
	int clocksource -   0=internal, 1=external
	int clockrate   -   Xilinx scanning frequency in Hertz
				                    legal values are:
				                    1,2,5,10,20,50,100 and 200
	int gain        -   gain for all ADC inputs
				        valid values are:
				        1,2,4,8,16,32,64 or 128
	int Vref        -   determine the internal ADC
				        reference voltage:
				        =1: 1.25V
				        =2: 2.5V
    Return:     None.

    Notes:

    Programmer(s):  Steve Hunt
                    Walter Scott            

**********************************************************/
int CONFIGURE(  int cardnum,
	            int vmeslotnum,
	            int ipslotnum,
	            int vectornum,
	            int aitype,
	            int IPmemsize,
	            int clocksource,
	            int clockrate,
	            int gain,
	            int Vref)
{
    /* SCO gain is HARDWIRED TO EIGHT HERE FOR TESTING */
    return intconfig(   cardnum,
                        vmeslotnum,
                        ipslotnum,
                        vectornum,
                        aitype,
		                IPmemsize,
                        clocksource,
                        clockrate,
                        gain,
                        Vref,
		                1,0,1,1,0,1,1);
}


/* Driver entry table */
struct{
  long number;
  long (*report)();
  long (*init)();
}DRV={
  2,
  IO_REPORT,
  DRIVER_INIT
};

