/********************************************************************************/
/*    H   H Y   Y TTTTT EEEEE  CCC       HYTEC ELECTRONICS LTD                  */
/*    H   H  Y Y    T   E     C          5 Cradock Road,                        */
/*    HHHHH   Y     T   EEE   C          Reading, Berks.    Tel: 0118 9757770   */
/*    H   H   Y     T   E     C          RG2 0JT            Fax: 0118 9757566   */
/*    H   H   Y     T   EEEEE  CCC       Web: www.hytec-electronics.co.uk       */
/********************************************************************************/

/* $Author: cross $ */
/* $Date: 2005/11/24 15:54:40 $ */
/* $Id: drvHy8403.h,v 1.2 2005/11/24 15:54:40 cross Exp $ */
/* $Name:  $ */
/* $Revision: 1.2 $ */

/********************************************************************************/
/*                                                                              */
/*  drvHy8403ip.h - Driver interface Header for Hytec 8403 ADC industry pack    */
/*                                                                              */
/*  (C)2004 Hytec Electronics Ltd.                                              */
/*                                                                              */
/********************************************************************************/
/*                                                                              */
/*  Revision history: (comment and initial revisions)                           */
/*                                                                              */
/*  vers.       revised         modified by                                     */
/*  -----       -----------     -------------------------                       */
/*  See Above.  21-NOV-2005     Darrell Nineham Hytec Electronics Ltd.          */
/*                              1. Add Thermocouple Calibrations Defines.       */
/*                                                                              */
/*  1.1.1.1     24-OCT-2005     Darrell Nineham Hytec Electronics Ltd.          */
/*                              1. Upgraded for Thermocouple Support.           */
/*                                                                              */
/*                                                                              */
/* ------------------- Copied from Non - Hot Swap Version --------------------  */
/*                                                                              */
/*                                                                              */
/*  1.4         21-OCT-2004     Darrell Nineham Hytec Electronics Ltd.          */
/*                              1. Upgraded to Hytec Coding Standard.           */
/*                              2. Copied from Non-Hot Swap Version.            */
/*                                                                              */
/*  1.3         27-JUL-2002     Steve Hunt                                      */
/*                              1. Last Non Hot Swap Version                    */
/*                                                                              */
/********************************************************************************/

#define Ok 0
#define ERR -1
#define EOS '\0'

#define STATIC static

/* define names of device support entry table symbols */
#define WF_DEV_SUP	devWfHy8403
#define AI_DEV_SUP	devAiHy8403

#define READ_WF		Hy8403_read_wave 
#define READ_AI		Hy8403_read_ai

/* driver initialisation define */
#define DRIVER_INIT Hy8403_driver_init
#define IO_REPORT   Hy8403_io_report
#define LIST        Hy8403List
#define CONFIGURE   Hy8403Configure
#define CALIBRATE   Hy8403ThermoCoupleCalibrate
#define WRITE_AO    Hy8403_write_ao


#define DRV         drvHy8403

/* READ_AI Offsets for Reading Calibration Values */
#define ADC_A_ZERO_CAL  16  /* ADC A - Zero Raw Calibration */
#define ADC_A_POS_CAL   17  /* ADC A - Pos  Raw Calibration */
#define ADC_A_NEG_CAL   18  /* ADC A - Neg  Raw Calibration */
#define ADC_B_ZERO_CAL  19  /* ADC B - Zero Raw Calibration */
#define ADC_B_POS_CAL   20  /* ADC B - Pos  Raw Calibration */
#define ADC_B_NEG_CAL   21  /* ADC B - Neg  Raw Calibration */

#include <dbScan.h>

int READ_WF(unsigned short card,
	    unsigned short channel,
	    char *parm,
	    unsigned long *val, 
	    unsigned long nelm
	    );

int READ_AI(unsigned short card,
	    unsigned short channel,
	    char *parm,
	    unsigned long *val
	    );


int WRITE_AO(unsigned short card,
	     unsigned short signal,
	     unsigned int val
	     );



int get8403ioscanpvt(unsigned short card, IOSCANPVT *scanpvt);

/************************************************************************/
/* * DRV driver error codes */
#define M_drvLib (1003<<16U)

#define drvError(CODE) (M_drvLib | (CODE))

#define S_drv_OK 0					/* success */
#define S_drv_badParam drvError(1)	/* driver: bad parameter*/
#define S_drv_noMemory drvError(2)	/* driver: no memory*/
#define S_drv_noDevice drvError(3)	/* driver: device not configured*/

#define S_drv_invSigMode drvError(4)
/* driver: signal mode conflicts with device config*/

#define S_drv_cbackChg drvError(5)
/* driver: specified callback differs from previous config*/

#define S_drv_alreadyQd drvError(6)
/* driver: a read request is already queued for the channel*/


