/********************************************************************************/
/*    H   H Y   Y TTTTT EEEEE  CCC       HYTEC ELECTRONICS LTD                  */
/*    H   H  Y Y    T   E     C          5 Cradock Road,                        */
/*    HHHHH   Y     T   EEE   C          Reading, Berks.    Tel: 0118 9757770   */
/*    H   H   Y     T   E     C          RG2 0JT            Fax: 0118 9757566   */
/*    H   H   Y     T   EEEEE  CCC       Web: www.hytec-electronics.co.uk       */
/********************************************************************************/

/* $Author: cross $ */
/* $Date: 2005/11/24 15:54:40 $ */
/* $Id: test8403.c,v 1.2 2005/11/24 15:54:40 cross Exp $ */
/* $Name:  $ */
/* $Revision: 1.2 $ */

/********************************************************************************/
/*                                                                              */
/*  test8403.c - A Set of Routines to Test Hytec 8403 ADC industry pack         */
/*                                                                              */
/*  (C)2004 Hytec Electronics Ltd.                                              */
/*                                                                              */
/********************************************************************************/
/*                                                                              */
/*  Revision history: (comment and initial revisions)                           */
/*                                                                              */
/*  vers.       revised         modified by                                     */
/*  -----       -----------     -------------------------                       */
/*  See Above.  21-NOV-2005     Darrell Nineham Hytec Electronics Ltd.          */                 
/*                              1. showthermocouple() to displays Calibration   */
/*                              Values.                                         */
/*                                                                              */    
/*  1.1.1.1     24-OCT-2005     Darrell Nineham Hytec Electronics Ltd.          */                 
/*                              1. Initial Version.                             */
/*                              2. Moved to Hy8403 Directory as now Hot Swap.   */
/*                              3. showall() added to display PTRs.             */
/*                              4. showthermocouple() added to display K-Type   */
/*                              Thermcouples.                                   */
/*                                                                              */    
/********************************************************************************/

/* ANSI standard */
#include <stdio.h>
#include <math.h>

/* OS specific vxworks */

#include <vxWorks.h>
#include <vxLib.h>
#include <vme.h>
#include <logLib.h>
#include <tyLib.h>
#include <sioLib.h>


#include "drvHy8403.h"


typedef unsigned short word;
    

/**********************************************************

    NAME:	showall()

    Description:

    Function - Reads ADC values from a 8913-P and displays
    them as hex and decimal.  Also calculates equivelant
    voltage and temperature.


    Parameters:

    Inputs:         card = number of IP card to display.
    Outputs:        None.
    Return:         None. 

    Notes:

    Programmer(s):  Darrell Nineham

**********************************************************

Revision History
----------------

Date            By      Version     Descripition
----            --      -------     ------------
24-OCT-2005     DAN                 Enhanced to accept
                                    passed card number.

01-NOV-2004     DAN                 Intial Version.
    
**********************************************************/
void showall(int card)
{
    word            iChannelCount;
    unsigned long   ADC_Value;
    float           Voltage;

    float           Temperature;
    float           Rref = 100; /* Reference/Vish Resistor is 100R */
    float           IrefA;
    float           IrefB;
    float           A = 0.00390802;
    float           B = -0.000000580195;
   
    /* Measure ADC A's Reference Voltage */
    Hy8403_read_ai( (word)(card),
                    (word)(7),
                    "SMOO",
                    &ADC_Value);
    
    /* Calculate ADC A's Common Current */
    Voltage = (float)(2.5 * ADC_Value) / (float)(16777216);
    IrefA = Voltage/Rref;

   
    /* Measure ADC B's Reference Volatage */
    Hy8403_read_ai( (word)(card),
                    (word)(15),
                    "SMOO",
                    &ADC_Value);
    
    /* Calculate ADC B's Common Current */
    Voltage = (float)(2.5 * ADC_Value) / (float)(16777216);
    IrefB = Voltage/Rref;


    /* Display ADC A */ 
    for(iChannelCount = 0; iChannelCount < 7; iChannelCount++)
    {
        Hy8403_read_ai( (word)(card),
                        iChannelCount,
                        "SMOO",
                        &ADC_Value);

                              
        Voltage = (float)(2.5 * ADC_Value) / (float)(16777216);
        
        Temperature = (Rref - Voltage/IrefA)/(-0.5 * (A*Rref + sqrt((A*A * Rref*Rref) - 4*B*Rref * (Rref - Voltage/IrefA))));                               
        
        printf("Channel %2d- 0x%08x - %8d - %1.6f V - %1.1fC\n",iChannelCount,
                                            (unsigned int)ADC_Value,
                                            (unsigned int)ADC_Value,
                                            Voltage/4,
                                            Temperature); 
    }
    
    
    /* Display ADC B */ 
    for(iChannelCount = 8; iChannelCount < 15; iChannelCount++)
    {
        Hy8403_read_ai( (word)(card),
                        iChannelCount,
                        "SMOO",
                        &ADC_Value);

                              
        Voltage = (float)(2.5 * ADC_Value) / (float)(16777216);
        
        Temperature = (Rref - Voltage/IrefB)/(-0.5 * (A*Rref + sqrt((A*A * Rref*Rref) - 4*B*Rref * (Rref - Voltage/IrefB))));                               
        
        printf("Channel %2d- 0x%08x - %8d - %1.6f V - %1.1fC\n",iChannelCount,
                                            (unsigned int)ADC_Value,
                                            (unsigned int)ADC_Value,
                                            Voltage/4,
                                            Temperature); 
    }
}



/**********************************************************

    NAME:	showthermocouple()

    Description:

    Function - Reads ADC values from a 8913-T and displays
    them as hex and decimal.  Also calculates equivelant
    voltage and temperature for K-Type Thermocouples.


    Parameters:

    Inputs:         card = number of IP card to display.
    Outputs:        None.
    Return:         None. 

    Notes:

    Programmer(s):  Darrell Nineham

**********************************************************

Revision History
----------------

Date            By      Version     Descripition
----            --      -------     ------------
21-NOV-2005     DAN     1.1.1.2     1. Displays Calibration
                                    Values.

24-OCT-2005     DAN     1.1.1.1     Intial Version.
    
**********************************************************/
void showthermocouple(int card)
{
    word            iChannelCount;
    unsigned long   ADC_Value;

	/* PT-100 / Cold Junction Temperature Variables */
    float			Vpt100;		/* PT100 Voltage */
    float			Ipt100;		/* PT100 Current */
    float			Rpt100;		/* PT100 Resistance */
    float			Tpt100;		/* PT100 Temperature */
    float           Rref = 100; /* Reference/Vish Resistor is 100R */
    float           A = 0.00390802;
    float           B = -0.000000580195;

	float			Gain = 64;
	
	float           Voltage;
    float           Temperature;
    float           TV;
   
    /* Measure PT100 / Cold Junction Temperature ADC B's Reference Voltage */
    Hy8403_read_ai( (word)(card),
                    (word)(15),
                    "SMOO",
                    &ADC_Value);

	/* Convert PT100 Value into Volts */
	Vpt100 = ((float)(ADC_Value) - 0x800000)*(float)(2.5/Gain) / (float)(16777216);

	/* Calculate Current through PT100 */
	Ipt100 = (float)(2.5 - Vpt100)/25000;

	/* Calculate Resistance of PT100 */
	Rpt100 = Vpt100/Ipt100;

	/* Calculate Temperature of PT100 */
	Tpt100 = (Rref - Vpt100/Ipt100)/(-0.5 * (A*Rref + sqrt((A*A * Rref*Rref) - 4*B*Rref * (Rref - Vpt100/Ipt100))));                               

    printf("PT100 - 0x%08x - %8d - %1.6f V - %1.6f A - %1.6f R -  %1.2fC\n",	(unsigned int)ADC_Value,
															                    (unsigned int)ADC_Value,
															                    Vpt100,
															                    Ipt100,
															                    Rpt100,
															                    Tpt100); 

    /* Convert PT100 Temperature to an equivalent K Type Thermocouple Voltage */
    TV =    (float)(-17.600413686) 
            + ((float)(38.921204975) * (float)(Tpt100)) 
            + ((float)( 0.018558770032) * (float)(Tpt100*Tpt100)) 
            + ((float)(-0.000099457592874) * (float)(Tpt100*Tpt100*Tpt100)) 
            + ((float)( 0.00000031840945719) * (float)(Tpt100*Tpt100*Tpt100*Tpt100)) 
            + ((float)( 0.00000000056072844889) * (float)(Tpt100*Tpt100*Tpt100*Tpt100*Tpt100)) 
            + ((float)( 0.00000000000056075059059) * (float)(Tpt100*Tpt100*Tpt100*Tpt100*Tpt100*Tpt100)) 
            + ((float)( 0.00000000000000032020720003) * (float)(Tpt100*Tpt100*Tpt100*Tpt100*Tpt100*Tpt100*Tpt100)) 
            + ((float)( 0.000000000000000000097151147152) * (float)(Tpt100*Tpt100*Tpt100*Tpt100*Tpt100*Tpt100*Tpt100*Tpt100)) 
            + ((float)( 0.000000000000000000000012104721275) * (float)(Tpt100*Tpt100*Tpt100*Tpt100*Tpt100*Tpt100*Tpt100*Tpt100*Tpt100)) 
            + ((float) 0.01185976 * exp(-0.0001183432 * (Tpt100-126.9686) * (Tpt100-126.9686))); 

    
    /* Display ADC A */ 

    for(iChannelCount = 0; iChannelCount < 7; iChannelCount++)
    {
        /* Read Calibrated Value from ADC */
        Hy8403_read_ai( (word)(card),
                        iChannelCount,
                        "SMOO",
                        &ADC_Value);

	    /* Convert Thermocouple Raw ADC Value into Volts */
	    Vpt100 = ((float)(ADC_Value) - 0x800000)*(float)(2.5/Gain) / (float)(16777216);
                              
        /* Add Cold Junction Voltage to Thermocouple Voltage and Convert to Microvolts (i.e. x 1000000) */
        Voltage = ((float)(TV / 1000000) + (float)(Vpt100)) * (float)(1000000);
        
        /* Convert Voltage to Temperature - Using K Type Formula assuming Temperature in Range 0-500C */
        /* The user may prefer to replace this with a Straight Line approximation or EPICS record Calculation */
        Temperature = ((float)( 0.0250835) * (float)(Voltage))
                    + ((float)( 0.00000007860106) * (float)(Voltage*Voltage))   
                    + ((float)(-0.000000000250313131) * (float)(Voltage*Voltage*Voltage))   
                    + ((float)( 0.0000000000000831527) * (float)(Voltage*Voltage*Voltage*Voltage))   
                    + ((float)( 0.00000000000000001228034) * (float)(Voltage*Voltage*Voltage*Voltage*Voltage));   


        printf("ADC A Channel %2d - Raw ADC - 0x%08x  Temp -  %1.1fC\n",   iChannelCount+1, 
                                                                            ADC_Value,
                                                                            Temperature); 
    }
    
    
    /* Display ADC B */ 

    for(iChannelCount = 8; iChannelCount < 14; iChannelCount++)
    {
        /* Read Calibrated Value from ADC */
        Hy8403_read_ai( (word)(card),
                        iChannelCount,
                        "SMOO",
                        &ADC_Value);

	    /* Convert Thermocouple Raw ADC Value into Volts */
	    Vpt100 = ((float)(ADC_Value) - 0x800000)*(float)(2.5/Gain) / (float)(16777216);
                              
        /* Add Cold Junction Voltage to Thermocouple Voltage and Convert to Microvolts (i.e. x 1000000) */
        Voltage = ((float)(TV / 1000000) + (float)(Vpt100)) * (float)(1000000);
        
        /* Convert Voltage to Temperature - Using K Type Formula assuming Temperature in Range 0-500C */
        /* The user may prefer to replace this with a Straight Line approximation or EPICS record Calculation */
        Temperature = ((float)( 0.0250835) * (float)(Voltage))
                    + ((float)( 0.00000007860106) * (float)(Voltage*Voltage))   
                    + ((float)(-0.000000000250313131) * (float)(Voltage*Voltage*Voltage))   
                    + ((float)( 0.0000000000000831527) * (float)(Voltage*Voltage*Voltage*Voltage))   
                    + ((float)( 0.00000000000000001228034) * (float)(Voltage*Voltage*Voltage*Voltage*Voltage));   


        printf("ADC B Channel %2d - Raw ADC - 0x%08x  Temp -  %1.1fC\n",   iChannelCount-7, 
                                                                            ADC_Value,
                                                                            Temperature); 
    }
        

    /* Display - Calibration Values - Can be Monitored for Alarms */

    /* Read Raw Value from ADC A - Zero Raw Calibration */
    Hy8403_read_ai( (word)(card),
                    ADC_A_ZERO_CAL,
                    "SMOO",
                    &ADC_Value);
	printf("ADC A - Zero Raw Calibration - 0x%08x\n",ADC_Value); 


    /* Read Raw Value from ADC A - Pos  Raw Calibration */
    Hy8403_read_ai( (word)(card),
                    ADC_A_POS_CAL,
                    "SMOO",
                    &ADC_Value);
	printf("ADC A - Pos  Raw Calibration - 0x%08x\n",ADC_Value); 


    /* Read Raw Value from ADC A - Neg  Raw Calibration */
    Hy8403_read_ai( (word)(card),
                    ADC_A_NEG_CAL,
                    "SMOO",
                    &ADC_Value);
	printf("ADC A - Neg  Raw Calibration - 0x%08x\n",ADC_Value); 


    /* Read Raw Value from ADC B - Zero Raw Calibration */
    Hy8403_read_ai( (word)(card),
                    ADC_B_ZERO_CAL,
                    "SMOO",
                    &ADC_Value);
	printf("ADC B - Zero Raw Calibration - 0x%08x\n",ADC_Value); 


    /* Read Raw Value from ADC B - Pos  Raw Calibration */
    Hy8403_read_ai( (word)(card),
                    ADC_B_POS_CAL,
                    "SMOO",
                    &ADC_Value);
	printf("ADC B - Pos  Raw Calibration - 0x%08x\n",ADC_Value); 


    /* Read Raw Value from ADC B - Neg  Raw Calibration */
    Hy8403_read_ai( (word)(card),
                    ADC_B_NEG_CAL,
                    "SMOO",
                    &ADC_Value);
	printf("ADC B - Neg  Raw Calibration - 0x%08x\n",ADC_Value); 
}
